
#define _CRT_SECURE_NO_DEPRECATE

#include <stdlib.h>
#include <string.h>
#include "md3_writer.h"

static const int LINE_LEN = 2048;


static const int SPECIES_COUNT = 118 + 1;

//OpenMXœ͂錳fL//
static const char* openmx_species_list[] = {
		"E",		//emptyO//
		"H", "He",	//1
		"Li", "Be",	"B", "C", "N", "O", "F", "Ne",		//2
		"Na", "Mg",	"Al", "Si", "P", "S", "Cl", "Ar",	//3
		"K", "Ca", "Sc", "Ti", "V", "Cr", "Mn", "Fe", "Co", "Ni", "Cu", "Zn", "Ga", "Ge", "As", "Se", "Br", "Kr",	//4
		"Rb", "Sr", "Y", "Zr", "Nb", "Mo", "Tc", "Ru", "Rh", "Pd", "Ag", "Cd", "In", "Sn", "Sb", "Te", "I", "Xe",	//5
		"Cs", "Ba",	//6(Cs, Ba)
		"La", "Ce", "Pr", "Nd", "Pm", "Sm", "Eu", "Gd", "Tb", "Dy", "Ho", "Er", "Tm", "Yb", "Lu",	////6(^mCh)
		"Hf", "Ta", "W", "Re", "Os", "Ir", "Pt", "Au", "Hg", "Tl", "Pb", "Bi", "Po", "At", "Rn",//6(Hf-Rn)
		"Fr", "Ra",	//7(Fr, Ra)
		"Ac", "Th", "Pa", "U", "Np", "Pu", "Am", "Cm", "Bk", "Cf", "Es", "Fm", "Md", "No", "Lr",	////7(AN`mCh)
		//104,105,106,107,108,109,110,111,112,113,114,115,116,117,118,//7(Rf-Uuo)
};


//fL̕񂩂猴qԍԂ//
inline static int GetAtomicNumber(const char* atomicsymbol){
	for(int i = 0; i < SPECIES_COUNT; i++){
		if(strcmp(atomicsymbol, openmx_species_list[i])==0){
			return i;
		}
	}
	return -1;
}


MD3_Writer::MD3_Writer() : 
		m_fp (NULL)
{
	
	
}

MD3_Writer::~MD3_Writer(){
	Close();
}


int MD3_Writer::Open(const char* filepath){
	
	if(m_fp){
		return -2;
	}

	m_fp = fopen(filepath, "w");
	if( ! m_fp){
		m_fp = NULL;
		return -1;
	}

	m_frameno=0;
	return 0;
}

void MD3_Writer::Close(){
	if(m_fp){
		fclose(m_fp);
		m_fp = NULL;
	}

}



int MD3_Writer::Write(int count, const int* knd, const double* r, const double* f, const double *boxaxis, const char* comment){

	if(m_fp == NULL){ return -1;}

	//q̏o//
	fprintf(m_fp, "%d\n", count);
	
	//Rgs̏o//
	if(comment){
		fprintf(m_fp, "%s\n", comment);
	}else{
		fprintf(m_fp, "frame = %d\n", m_frameno);
	}

	//BOX̏o//
	if(boxaxis){
		fprintf(m_fp,"BOX   %15.8f  %15.8f  %15.8f  %15.8f  %15.8f  %15.8f  %15.8f  %15.8f  %15.8f\n",
			boxaxis[0], boxaxis[1], boxaxis[2], 
			boxaxis[3], boxaxis[4], boxaxis[5],
			boxaxis[6], boxaxis[7], boxaxis[8]);
	}else{
		fprintf(m_fp,"BOX   %15.8f  %15.8f  %15.8f  %15.8f  %15.8f  %15.8f  %15.8f  %15.8f  %15.8f\n",
			10.0, 0.0, 0.0, 
			0.0, 10.0, 0.0,
			0.0, 0.0, 10.0);
	}
	
	if(f){
		for( int i = 0; i < count; i++){
			fprintf(m_fp,"%4s   %15.8f  %15.8f  %15.8f  %15.8f  %15.8f  %15.8f\n",
                  openmx_species_list[knd[i]],
                  r[i*3], r[i*3+1], r[i*3+2],
                  f[i*3], f[i*3+1], f[i*3+2]);
		}
	}else{
		for( int i = 0; i < count; i++){
			fprintf(m_fp,"%4s   %15.8f  %15.8f  %15.8f  %15.8f  %15.8f  %15.8f\n",
                  openmx_species_list[knd[i]],
                  r[i*3], r[i*3+1], r[i*3+2],
                  0.0, 0.0, 0.0);
		}

	}

	m_frameno++;

	return m_frameno;
}



int MD3_Writer::Write_f(int count, const int* knd, const float* r, const float* f, const float *boxaxis, const char* comment){

	if(m_fp == NULL){ return -1;}

	//q̏o//
	fprintf(m_fp, "%d\n", count);
	
	//Rgs̏o//
	if(comment){
		fprintf(m_fp, "%s\n", comment);
	}else{
		fprintf(m_fp, "frame = %d\n", m_frameno);
	}

	//BOX̏o//
	if(boxaxis){
		fprintf(m_fp,"BOX   %15.8f  %15.8f  %15.8f  %15.8f  %15.8f  %15.8f  %15.8f  %15.8f  %15.8f\n",
			boxaxis[0], boxaxis[1], boxaxis[2], 
			boxaxis[3], boxaxis[4], boxaxis[5],
			boxaxis[6], boxaxis[7], boxaxis[8]);
	}else{
		fprintf(m_fp,"BOX   %15.8f  %15.8f  %15.8f  %15.8f  %15.8f  %15.8f  %15.8f  %15.8f  %15.8f\n",
			10.0, 0.0, 0.0, 
			0.0, 10.0, 0.0,
			0.0, 0.0, 10.0);
	}
	
	if(f){
		for( int i = 0; i < count; i++){
			fprintf(m_fp,"%4s   %15.8f  %15.8f  %15.8f  %15.8f  %15.8f  %15.8f\n",
                  openmx_species_list[knd[i]],
                  r[i*3], r[i*3+1], r[i*3+2],
                  f[i*3], f[i*3+1], f[i*3+2]);
		}
	}else{
		for( int i = 0; i < count; i++){
			fprintf(m_fp,"%4s   %15.8f  %15.8f  %15.8f  %15.8f  %15.8f  %15.8f\n",
                  openmx_species_list[knd[i]],
                  r[i*3], r[i*3+1], r[i*3+2],
                  0.0, 0.0, 0.0);
		}

	}

	m_frameno++;

	return m_frameno;
}
