
/////!=====================================================================;
////// REBO(Reactive Empirical Bond Order) potential by D.W. Brenner at 2002;
////// include energy current calculation.;
////// fastest version.;
////// ;
////// ;
////// ver. 20070418;
////// ;
/////!=====================================================================;

#include "rebojf.h"
#include <string.h>
	

REBO::REBO(int maxcount) : CellDiv(2.0, FLAG_PERIODIC_XYZ, maxcount){

	m_ijmax = maxcount * BOND_CNT;

	m_Dmax[0] = 2.0;
	m_Dmax[1] = 1.8;
	m_Dmax[2] = 1.7;

	m_Dmin[0] = 1.7;
	m_Dmin[1] = 1.3;
	m_Dmin[2] = 1.1;

		m_Dmax2[0] = m_Dmax[0] * m_Dmax[0];
		m_Dmax2[1] = m_Dmax[1] * m_Dmax[1];
		m_Dmax2[2] = m_Dmax[2] * m_Dmax[2];

		m_Ddiff[0] = m_Dmax[0] - m_Dmin[0];
		m_Ddiff[1] = m_Dmax[1] - m_Dmin[1];
		m_Ddiff[2] = m_Dmax[2] - m_Dmin[2];

	m_rp = new REBO_PARTICLE[m_max_count];

	m_bonding = new REBO_BONDING[m_ijmax];
	m_Jij = new double[m_ijmax];

	

	m_bondCnt = 0;

	m_Fr_v.Set(0.0, 0.0, 0.0);
	
	InitSpline();

}

REBO::~REBO(){
	delete [] m_rp;
	delete [] m_bonding;
	delete [] m_Jij;

}

void REBO::SetParticlePointers(AtomArray* atoms){
	m_knd = atoms->knd;
	m_r = atoms->r;
	m_p = atoms->p;
	m_f = atoms->f;
	m_mass = atoms->mass;
	m_pot = atoms->pot;
	
}

void REBO::InitSpline(){
		

	// set zero;
		memset(g_spP, 0, (2)*(SP_PMAXY+1)*(SP_PMAXX+1)*sizeof(double));
		memset(g_spF, 0, (3)*(SP_FMAXZ+1)*(SP_FMAXY+1)*(SP_FMAXX+1)*sizeof(double));
		memset(g_spFdx, 0, (SP_FMAXZ+1)*(SP_FMAXY+1)*(SP_FMAXX+1)*sizeof(double));
		memset(g_spFdy, 0, (SP_FMAXZ+1)*(SP_FMAXY+1)*(SP_FMAXX+1)*sizeof(double));
		memset(g_spFdz, 0, (SP_FMAXZ+1)*(SP_FMAXY+1)*(SP_FMAXX+1)*sizeof(double));
		memset(g_spT, 0, (SP_TMAXZ+1)*(SP_TMAXY+1)*(SP_TMAXX+1)*sizeof(double));
		

	// bicubic spline < Pcc > ------------;
		g_spP[1][1][KIND_C] = 0.003026697473481;	//(CH3)HC=CH(CH3);
		g_spP[2][0][KIND_C] = 0.007860700254745;	//C2H4;
		g_spP[3][0][KIND_C] = 0.016125364564267;	//C2H6;
		g_spP[1][2][KIND_C] = 0.003179530830731;	//i-C4H10;
		g_spP[2][1][KIND_C] = 0.006326248241119;	//c-C6H12;
		;
	// bicubic spline < Pch > ------------;
		g_spP[1][0][KIND_H] = 0.2093367328250380;	//CH2;
		g_spP[2][0][KIND_H] =-0.064449615432525;	//CH3;
		g_spP[3][0][KIND_H] =-0.303927546346162;	//CH4;
		g_spP[0][1][KIND_H] = 0.01;				//C2H2;
		g_spP[0][2][KIND_H] =-0.1220421462782555;	//(CH3)HC=CH(CH3);
		g_spP[1][1][KIND_H] =-0.1251234006287090;	//C2H4;
		g_spP[2][1][KIND_H] =-0.298905245783;		//C2H6;
		g_spP[0][3][KIND_H] =-0.307584705066;		//i-C4H10;
		g_spP[1][2][KIND_H] =-0.3005291724067579;	//c-C6H12;
	;
	// tricubic spline <Fcc> -----------------;
		g_spF[1][1][1][KIND_CC] = 0.105000;			//Acetylene;
		g_spF[1][1][2][KIND_CC] =-0.0041775;			//H2C=C=CH;
		g_spF[1][1][3][KIND_CC] =-0.0160856;			//C4;
		g_spF[1][1][4][KIND_CC] =-0.0160856;			//C4;
		g_spF[1][1][5][KIND_CC] =-0.0160856;			//C4;
		g_spF[2][2][1][KIND_CC] = 0.09444957;			//(CH3]2C=C(CH3]2;
		g_spF[2][2][2][KIND_CC] = 0.04632351;			//0.02200000;			//Benzene;
		g_spF[2][2][3][KIND_CC] = 0.03088234;			//Average from difference F(2,2,2] to difference F(2,2,9];
		g_spF[2][2][4][KIND_CC] = 0.01544117;			//Average from difference F(2,2,2] to difference F(2,2,9];
		//g_spF[2][2][2][KIND_CC] = 0.02200000;//0.04632351;			//0.02200000;			//Benzene;
		//g_spF[2][2][3][KIND_CC] = 0.01466666;			//Average from difference F(2,2,2] to difference F(2,2,9];
		//g_spF[2][2][4][KIND_CC] = 0.00733333;			//Average from difference F(2,2,2] to difference F(2,2,9];
		g_spF[2][2][5][KIND_CC] = 0.0;					//graphite;
		
		g_spF[0][1][1][KIND_CC] = 0.04338699;			//C2H;
		g_spF[0][1][2][KIND_CC] = 0.0099172158;		//C3;
		g_spF[0][2][1][KIND_CC] = 0.0493976637;		//CCH2;
		g_spF[0][2][2][KIND_CC] =-0.011942669;			//CCH(CH2];
		g_spF[0][3][1][KIND_CC] =-0.119798935;		//H3CC;
		g_spF[0][3][2][KIND_CC] =-0.119798935;		//H3CC;
		g_spF[0][3][3][KIND_CC] =-0.119798935;		//H3CC;
		g_spF[0][3][4][KIND_CC] =-0.119798935;		//H3CC;
		g_spF[0][3][5][KIND_CC] =-0.119798935;		//H3CC;
		g_spF[1][2][1][KIND_CC] = 0.0096495698;		//H2CCH;
		g_spF[1][2][2][KIND_CC] = 0.030;				//H2C=C=CH2;
		g_spF[1][2][3][KIND_CC] =-0.0200;				//C6H5;
		g_spF[1][2][4][KIND_CC] =-0.030133632;		//Graphite vacancy;
		g_spF[1][2][5][KIND_CC] =-0.030133632;		//Graphite vacancy;
		g_spF[1][3][2][KIND_CC] =-0.124836752;		//H3C-CCH;
		g_spF[1][3][3][KIND_CC] =-0.124836752;		//H3C-CCH;
		g_spF[1][3][4][KIND_CC] =-0.124836752;		//H3C-CCH;
		g_spF[1][3][5][KIND_CC] =-0.124836752;		//H3C-CCH;
		g_spF[2][3][1][KIND_CC] =-0.044709383;		//Diamond vacancy;
		g_spF[2][3][2][KIND_CC] =-0.044709383;		//Diamond vacancy;
		g_spF[2][3][3][KIND_CC] =-0.044709383;		//Diamond vacancy;
		g_spF[2][3][4][KIND_CC] =-0.044709383;		//Diamond vacancy;
		g_spF[2][3][5][KIND_CC] =-0.044709383;		//Diamond vacancy;
		
			g_spF[1][0][1][KIND_CC] = 0.04338699;			//C2H;
			g_spF[1][0][2][KIND_CC] = 0.0099172158;		//C3;
			g_spF[2][0][1][KIND_CC] = 0.0493976637;		//CCH2;
			g_spF[2][0][2][KIND_CC] =-0.011942669;			//CCH(CH2];
			g_spF[3][0][1][KIND_CC] =-0.119798935;		//H3CC;
			g_spF[3][0][2][KIND_CC] =-0.119798935;		//H3CC;
			g_spF[3][0][3][KIND_CC] =-0.119798935;		//H3CC;
			g_spF[3][0][4][KIND_CC] =-0.119798935;		//H3CC;
			g_spF[3][0][5][KIND_CC] =-0.119798935;		//H3CC;
			g_spF[2][1][1][KIND_CC] = 0.0096495698;		//H2CCH;
			g_spF[2][1][2][KIND_CC] = 0.030;				//H2C=C=CH2;
			g_spF[2][1][3][KIND_CC] =-0.0200;				//C6H5;
			g_spF[2][1][4][KIND_CC] =-0.030133632;		//Graphite vacancy;
			g_spF[2][1][5][KIND_CC] =-0.030133632;		//Graphite vacancy;
			g_spF[3][1][2][KIND_CC] =-0.124836752;		//H3C-CCH;
			g_spF[3][1][3][KIND_CC] =-0.124836752;		//H3C-CCH;
			g_spF[3][1][4][KIND_CC] =-0.124836752;		//H3C-CCH;
			g_spF[3][1][5][KIND_CC] =-0.124836752;		//H3C-CCH;
			g_spF[3][2][1][KIND_CC] =-0.044709383;		//Diamond vacancy;
			g_spF[3][2][2][KIND_CC] =-0.044709383;		//Diamond vacancy;
			g_spF[3][2][3][KIND_CC] =-0.044709383;		//Diamond vacancy;
			g_spF[3][2][4][KIND_CC] =-0.044709383;		//Diamond vacancy;
			g_spF[3][2][5][KIND_CC] =-0.044709383;		//Diamond vacancy;
		
		g_spFdx[2][1][1] =-0.052500;;
		g_spFdx[2][1][3] =-0.054376;;
		g_spFdx[2][1][4] =-0.054376;;
		g_spFdx[2][1][5] =-0.054376;;
		g_spFdx[2][3][1] = 0.0;;
		g_spFdx[2][3][2] = 0.062418;;
		g_spFdx[2][3][3] = 0.062418;;
		g_spFdx[2][3][4] = 0.062418;;
		g_spFdx[2][3][5] = 0.062418;;
		
			g_spFdy[1][2][1] =-0.052500;;
			g_spFdy[1][2][3] =-0.054376;;
			g_spFdy[1][2][4] =-0.054376;;
			g_spFdy[1][2][5] =-0.054376;;
			g_spFdy[3][2][1] = 0.0;;
			g_spFdy[3][2][2] = 0.062418;;
			g_spFdy[3][2][3] = 0.062418;;
			g_spFdy[3][2][4] = 0.062418;;
			g_spFdy[3][2][5] = 0.062418;;
			
			
		g_spFdz[2][2][4] =-0.006618;;
		g_spFdz[1][1][2] =-0.060543;;
		g_spFdz[1][2][3] =-0.020044;;
		
			g_spFdz[2][1][3] =-0.020044;;
	
	// tricubic spline <Fhh> -----------------;
		g_spF[1][1][1][KIND_HH] = 0.249831916;;
	
	// tricubic spline <Fch> -----------------;
		g_spF[0][2][3][KIND_CH] =-0.0090477875161288110;		//C6H6;
		g_spF[0][2][4][KIND_CH] =-0.0090477875161288110;		//C6H6;
		g_spF[0][2][5][KIND_CH] =-0.0090477875161288110;		//C6H6;

		g_spF[1][3][1][KIND_CH] =-0.213;						//Equations[23]-[25];
		g_spF[1][3][2][KIND_CH] =-0.213;						//Equations[23]-[25];
		g_spF[1][3][3][KIND_CH] =-0.213;						//Equations[23]-[25];
		g_spF[1][3][4][KIND_CH] =-0.213;						//Equations[23]-[25];
		g_spF[1][3][5][KIND_CH] =-0.213;						//Equations[23]-[25];
		g_spF[1][2][1][KIND_CH] =-0.25;						//Equations[23]-(25];
		g_spF[1][2][2][KIND_CH] =-0.25;						//Equations[23]-(25];
		g_spF[1][2][3][KIND_CH] =-0.25;						//Equations[23]-(25];
		g_spF[1][2][4][KIND_CH] =-0.25;						//Equations[23]-(25];
		g_spF[1][2][5][KIND_CH] =-0.25;						//Equations[23]-(25];
		g_spF[1][1][1][KIND_CH] =-0.5;						//Equations(23]-(25];
		g_spF[1][1][2][KIND_CH] =-0.5;						//Equations(23]-(25];
		g_spF[1][1][3][KIND_CH] =-0.5;						//Equations(23]-(25];
		g_spF[1][1][4][KIND_CH] =-0.5;						//Equations(23]-(25];
		g_spF[1][1][5][KIND_CH] =-0.5;						//Equations(23]-(25];
			
			g_spF[2][0][3][KIND_CH] =-0.0090477875161288110;		//C6H6;
			g_spF[2][0][4][KIND_CH] =-0.0090477875161288110;		//C6H6;
			g_spF[2][0][5][KIND_CH] =-0.0090477875161288110;		//C6H6;
			g_spF[3][1][1][KIND_CH] =-0.213;						//Equations(23]-(25];
			g_spF[3][1][2][KIND_CH] =-0.213;						//Equations(23]-(25];
			g_spF[3][1][3][KIND_CH] =-0.213;						//Equations(23]-(25];
			g_spF[3][1][4][KIND_CH] =-0.213;						//Equations(23]-(25];
			g_spF[3][1][5][KIND_CH] =-0.213;						//Equations(23]-(25];
			g_spF[2][1][1][KIND_CH] =-0.25;						//Equations(23]-(25];
			g_spF[2][1][2][KIND_CH] =-0.25;						//Equations(23]-(25];
			g_spF[2][1][3][KIND_CH] =-0.25;						//Equations(23]-(25];
			g_spF[2][1][4][KIND_CH] =-0.25;						//Equations(23]-(25];
			g_spF[2][1][5][KIND_CH] =-0.25;						//Equations(23]-(25];
	;
	// tricubic spline <Tcc> -----------------;
		g_spT[2][2][1] =-0.070280085;		//Ethene;
		g_spT[2][2][5] =-0.00809675;		//Solid-state structure;
		
		//near nature ??;
		//g_spT[2][2][4] =-0.0178;;
		//g_spT[2][2][3] =-0.01009675;;
}


inline void REBO::Reset(int count){
		memset(m_rp, 0, count * sizeof(REBO_PARTICLE));
		memset(m_Jij, 0, m_bondCnt * sizeof(double));
}


void REBO::EJSum(double dt){
		
	int i;
	
	for(i = 0; i < m_bondCnt; i++){
		m_Jij[i] *= dt;
	}
}

//======================================================================;
//;
//		Subroutine SetList (olg name is BrnList);
//;
//		This routine dig up interaction partner and calculate ;
//		coordination.;
//;
//;
//									made by Atsushi I.	2004/12/20;
//======================================================================;


//void REBO::SetListOrtho(double boxsize.x, double boxsize.y, double boxsize.z, CELLDIV_INFO* info){
void REBO::SetListOrtho(int count){
	
	// local variable ------------------------------;
	int i,j, jhead;
	int ki;
	vec3d dd;
	int bondcount = 0;

	//ϐ̃Zbg.
	Reset(count);

	//qZɋ؂.
	PutIntoCell(count, m_r, m_knd);

	

//Zł̒T==============================================
	int icell;
	for( icell=0; icell < m_mcell; icell++){
			//loop over all molecules in the cell;
				i = m_first_in_cell[icell];
				while (i != -1){

					ki = m_knd[i];
					j = m_inext[i];
	
			
					//nabor loop ----;
					while (j != -1){

						dd = m_r[i] - m_r[j];
					//	bondcount += AddBondByCutOff(i,j,ki,bondcount,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
						bondcount += AddBondByCutOff(i,j,ki,m_knd[j],bondcount,dd);	//bondǉꂽ1Ԃ.

						j = m_inext[j];
					}
					
					i = m_inext[i];		
				}

	}

//ʂ̃ZԂł̒T==============================================
	//E1cell.
	//Eɂ܂肽݂̔B
	int index;
	for( index=0; index < m_cell_pair_index[1]; index += 2){
		
		i = m_first_in_cell[m_cell_pair[index]];
		jhead = m_first_in_cell[m_cell_pair[index+1]];
		while (i != -1){

			ki = m_knd[i];
			//nabor loop ----;
			j = jhead;
			while (j != -1){
				dd = m_r[i] - m_r[j];
			//	bondcount += AddBondByCutOff(i,j,ki,bondcount,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
				bondcount += AddBondByCutOff(i,j,ki,m_knd[j],bondcount,dd);	//bondǉꂽ1Ԃ.

				j = m_inext[j];
			}// end of nabor loop;

			i = m_inext[i];
		}
	}

//xEׂ(jxzcell)
	for( index = m_cell_pair_index[1]; index < m_cell_pair_index[2]; index += 2){
		
		
		i = m_first_in_cell[m_cell_pair[index]];
		jhead = m_first_in_cell[m_cell_pair[index+1]];
		while (i != -1){

			ki = m_knd[i];
			//nabor loop ----;
			j = jhead;
			while (j != -1){
				dd = m_r[i] - m_r[j];
				dd.x -= m_box.x;
				//bondcount += AddBondByCutOff(i,j,ki,bondcount,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
				bondcount += AddBondByCutOff(i,j,ki,m_knd[j],bondcount,dd);	//bondǉꂽ1Ԃ.

				j = m_inext[j];
			}// end of nabor loop;

			i = m_inext[i];
		}
	}

//yEׂ(jyzcell)
	for( index = m_cell_pair_index[2]; index < m_cell_pair_index[3]; index += 2){
		
		
		i = m_first_in_cell[m_cell_pair[index]];
		jhead = m_first_in_cell[m_cell_pair[index+1]];
		while (i != -1){

			ki = m_knd[i];
			//nabor loop ----;
			j = jhead;
			while (j != -1){
				//GetRij_periodicY(i,j, &xx, &yy, &zz);
				dd = m_r[i] - m_r[j];
				dd.y -= m_box.y;
				//bondcount += AddBondByCutOff(i,j,ki,bondcount,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
				bondcount += AddBondByCutOff(i,j,ki,m_knd[j],bondcount,dd);	//bondǉꂽ1Ԃ.

				j = m_inext[j];
			}// end of nabor loop;

			i = m_inext[i];
		}
	}

//zEׂ(jzzcell)
	for( index = m_cell_pair_index[3]; index < m_cell_pair_index[4]; index += 2){
		
		
		i = m_first_in_cell[m_cell_pair[index]];
		jhead = m_first_in_cell[m_cell_pair[index+1]];
		while (i != -1){

			ki = m_knd[i];
			//nabor loop ----;
			j = jhead;
			while (j != -1){
				//GetRij_periodicZ(i,j, &xx, &yy, &zz);
				dd = m_r[i] - m_r[j];
				dd.z -= m_box.z;
				//bondcount += AddBondByCutOff(i,j,ki,bondcount,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
				bondcount += AddBondByCutOff(i,j,ki,m_knd[j],bondcount,dd);	//bondǉꂽ1Ԃ.

				j = m_inext[j];
			}// end of nabor loop;

			i = m_inext[i];
		}
	}

//xyEׂ(҂xyɉzcell)
	for( index = m_cell_pair_index[4]; index < m_cell_pair_index[5]; index += 2){
		
		i = m_first_in_cell[m_cell_pair[index]];
		jhead = m_first_in_cell[m_cell_pair[index+1]];
		while (i != -1){

			ki = m_knd[i];
			//nabor loop ----;
			j = jhead;
			while (j != -1){
				//GetRij_periodicXY(i,j, &xx, &yy, &zz);
				//bondcount += AddBondByCutOff(i,j,ki,bondcount,xx,yy,zz);	//bondǉꂽ1Ԃ.
				dd = m_r[i] - m_r[j];
				dd.x -= m_box.x;
				dd.y -= m_box.y;
				//bondcount += AddBondByCutOff(i,j,ki,bondcount,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
				bondcount += AddBondByCutOff(i,j,ki,m_knd[j],bondcount,dd);	//bondǉꂽ1Ԃ.

				j = m_inext[j];
			}// end of nabor loop;

			i = m_inext[i];
		}
	}
//xyEׂ(O҂xzcell,҂yzcell)
	for( index = m_cell_pair_index[5]; index < m_cell_pair_index[6]; index += 2){
		
		i = m_first_in_cell[m_cell_pair[index]];
		jhead = m_first_in_cell[m_cell_pair[index+1]];
		while (i != -1){

			ki = m_knd[i];
			//nabor loop ----;
			j = jhead;
			while (j != -1){
				//GetRij_periodicXY(i,j, &xx, &yy, &zz);
				//bondcount += AddBondByCutOff(i,j,ki,bondcount,xx,yy,zz);	//bondǉꂽ1Ԃ.
				dd = m_r[i] - m_r[j];
				dd.x += m_box.x;
				dd.y -= m_box.y;
				//bondcount += AddBondByCutOff(i,j,ki,bondcount,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
				bondcount += AddBondByCutOff(i,j,ki,m_knd[j],bondcount,dd);	//bondǉꂽ1Ԃ.

				j = m_inext[j];
			}// end of nabor loop;

			i = m_inext[i];
		}
	}

//yzEׂ(҂yzɉzcell)
	for( index = m_cell_pair_index[6]; index < m_cell_pair_index[7]; index += 2){
		
		i = m_first_in_cell[m_cell_pair[index]];
		jhead = m_first_in_cell[m_cell_pair[index+1]];
		while (i != -1){

			ki = m_knd[i];
			//nabor loop ----;
			j = jhead;
			while (j != -1){
				//GetRij_periodicYZ(i,j, &xx, &yy, &zz);
				//bondcount += AddBondByCutOff(i,j,ki,bondcount,xx,yy,zz);	//bondǉꂽ1Ԃ.
				dd = m_r[i] - m_r[j];
				dd.y -= m_box.y;
				dd.z -= m_box.z;
				//bondcount += AddBondByCutOff(i,j,ki,bondcount,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
				bondcount += AddBondByCutOff(i,j,ki,m_knd[j],bondcount,dd);	//bondǉꂽ1Ԃ.

				j = m_inext[j];
			}// end of nabor loop;

			i = m_inext[i];
		}
	}
//yzEׂ(O҂yzcell,҂zzcell)
	for( index = m_cell_pair_index[7]; index < m_cell_pair_index[8]; index += 2){
		
		i = m_first_in_cell[m_cell_pair[index]];
		jhead = m_first_in_cell[m_cell_pair[index+1]];
		while (i != -1){

			ki = m_knd[i];
			//nabor loop ----;
			j = jhead;
			while (j != -1){
				//GetRij_periodicYZ(i,j, &xx, &yy, &zz);
				//bondcount += AddBondByCutOff(i,j,ki,bondcount,xx,yy,zz);	//bondǉꂽ1Ԃ.
				dd = m_r[i] - m_r[j];
				dd.y += m_box.y;
				dd.z -= m_box.z;
				//bondcount += AddBondByCutOff(i,j,ki,bondcount,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
				bondcount += AddBondByCutOff(i,j,ki,m_knd[j],bondcount,dd);	//bondǉꂽ1Ԃ.

				j = m_inext[j];
			}// end of nabor loop;

			i = m_inext[i];
		}
	}
	
	
//zxEׂ(҂zxɉzcell)
	for( index = m_cell_pair_index[8]; index < m_cell_pair_index[9]; index += 2){
		
		i = m_first_in_cell[m_cell_pair[index]];
		jhead = m_first_in_cell[m_cell_pair[index+1]];
		while (i != -1){

			ki = m_knd[i];
			//nabor loop ----;
			j = jhead;
			while (j != -1){
				//GetRij_periodicZX(i,j, &xx, &yy, &zz);
				//bondcount += AddBondByCutOff(i,j,ki,bondcount,xx,yy,zz);	//bondǉꂽ1Ԃ.
				dd = m_r[i] - m_r[j];
				dd.x -= m_box.x;
				dd.z -= m_box.z;
				//bondcount += AddBondByCutOff(i,j,ki,bondcount,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
				bondcount += AddBondByCutOff(i,j,ki,m_knd[j],bondcount,dd);	//bondǉꂽ1Ԃ.

				j = m_inext[j];
			}// end of nabor loop;

			i = m_inext[i];
		}
	}
//zxEׂ(O҂zzcell,҂xzcell)
	for( index = m_cell_pair_index[9]; index < m_cell_pair_index[10]; index += 2){
		
		i = m_first_in_cell[m_cell_pair[index]];
		jhead = m_first_in_cell[m_cell_pair[index+1]];
		while (i != -1){

			ki = m_knd[i];
			//nabor loop ----;
			j = jhead;
			while (j != -1){
				//GetRij_periodicZX(i,j, &xx, &yy, &zz);
				//bondcount += AddBondByCutOff(i,j,ki,bondcount,xx,yy,zz);	//bondǉꂽ1Ԃ.
				dd = m_r[i] - m_r[j];
				dd.x -= m_box.x;
				dd.z += m_box.z;
				//bondcount += AddBondByCutOff(i,j,ki,bondcount,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
				bondcount += AddBondByCutOff(i,j,ki,m_knd[j],bondcount,dd);	//bondǉꂽ1Ԃ.

				j = m_inext[j];
			}// end of nabor loop;

			i = m_inext[i];
		}
	}

//xyzEׂ(O҂xzcell,҂yzzcell)
	index = m_cell_pair_index[10];
	if(index < m_cell_pair_index[11]){
		
		i = m_first_in_cell[m_cell_pair[index]];
		jhead = m_first_in_cell[m_cell_pair[index+1]];
		while (i != -1){

			ki = m_knd[i];
			//nabor loop ----;
			j = jhead;
			while (j != -1){
				//GetRij(i,j, &xx, &yy, &zz);
				//bondcount += AddBondByCutOff(i,j,ki,bondcount,xx,yy,zz);	//bondǉꂽ1Ԃ.
				dd = m_r[i] - m_r[j];
				dd.x += m_box.x;
				dd.y -= m_box.y;
				dd.z -= m_box.z;
				//bondcount += AddBondByCutOff(i,j,ki,bondcount,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
				bondcount += AddBondByCutOff(i,j,ki,m_knd[j],bondcount,dd);	//bondǉꂽ1Ԃ.

				j = m_inext[j];
			}// end of nabor loop;

			i = m_inext[i];
		}
	
	//(O҂yzcell,҂zxzcell)
	index +=2;
	
		i = m_first_in_cell[m_cell_pair[index]];
		jhead = m_first_in_cell[m_cell_pair[index+1]];
		while (i != -1){

			ki = m_knd[i];
			//nabor loop ----;
			j = jhead;
			while (j != -1){
				//GetRij(i,j, &xx, &yy, &zz);
				//bondcount += AddBondByCutOff(i,j,ki,bondcount,xx,yy,zz);	//bondǉꂽ1Ԃ.
				dd = m_r[i] - m_r[j];
				dd.x -= m_box.x;
				dd.y += m_box.y;
				dd.z -= m_box.z;
				//bondcount += AddBondByCutOff(i,j,ki,bondcount,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
				bondcount += AddBondByCutOff(i,j,ki,m_knd[j],bondcount,dd);	//bondǉꂽ1Ԃ.

				j = m_inext[j];
			}// end of nabor loop;

			i = m_inext[i];
		}
	//(O҂zzcell,҂xyzcell)
	index +=2;
	
		i = m_first_in_cell[m_cell_pair[index]];
		jhead = m_first_in_cell[m_cell_pair[index+1]];
		while (i != -1){

			ki = m_knd[i];
			//nabor loop ----;
			j = jhead;
			while (j != -1){
				//GetRij(i,j, &xx, &yy, &zz);
				//bondcount += AddBondByCutOff(i,j,ki,bondcount,xx,yy,zz);	//bondǉꂽ1Ԃ.
				dd = m_r[i] - m_r[j];
				dd.x -= m_box.x;
				dd.y -= m_box.y;
				dd.z += m_box.z;
				//bondcount += AddBondByCutOff(i,j,ki,bondcount,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
				bondcount += AddBondByCutOff(i,j,ki,m_knd[j],bondcount,dd);	//bondǉꂽ1Ԃ.

				j = m_inext[j];
			}// end of nabor loop;

			i = m_inext[i];
		}
	
	//(҂xyzzcell)
	index +=2;
	
		i = m_first_in_cell[m_cell_pair[index]];
		jhead = m_first_in_cell[m_cell_pair[index+1]];
		while (i != -1){

			ki = m_knd[i];
			//nabor loop ----;
			j = jhead;
			while (j != -1){
				//GetRij(i,j, &xx, &yy, &zz);
				//bondcount += AddBondByCutOff(i,j,ki,bondcount,xx,yy,zz);	//bondǉꂽ1Ԃ.
				dd = m_r[i] - m_r[j];
				dd.x -= m_box.x;
				dd.y -= m_box.y;
				dd.z -= m_box.z;
				//bondcount += AddBondByCutOff(i,j,ki,bondcount,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
				bondcount += AddBondByCutOff(i,j,ki,m_knd[j],bondcount,dd);	//bondǉꂽ1Ԃ.

				j = m_inext[j];
			}// end of nabor loop;

			i = m_inext[i];
		}
	
	}



	m_bondCnt = bondcount;

	
}

void REBO::SetListNonortho(mat33d Am, CELLDIV_INFO* info){
	
	// local variable ------------------------------;
		int i,j,l, jhead;
		//double xx,yy,zz;
		int ki;
		vec3d dd;
		vec3d peri;
		int* first_in_cell = info->first_in_cell;
		int* cell_pair = info->cell_pair;
		int* inext = info->inext;

	//ϐ̃Zbg.
	//Reset(count);
		
	//qZɋ؂.
	//PutIntoCell(count, m_r, m_knd);

		l = 0;


	//Zł̒T==============================================
		int icell;
		for( icell=0; icell < info->cellall; icell++){
				//loop over all molecules in the cell;
					i = first_in_cell[icell];
					while (i != -1){

						ki = m_knd[i];
						j = inext[i];
		
				
						//nabor loop ----;
						while (j != -1){

							dd = m_r[i] - m_r[j];
							l += AddBondByCutOff(i,j,ki,m_knd[j],l,dd);	//bondǉꂽ1Ԃ.

							j = inext[j];
						}
						
						i = inext[i];		
					}

		}

	//ʂ̃ZԂł̒T==============================================
		//E1cell.
		//Eɂ܂肽݂̔B
		int index;
		for( index=0; index < info->cell_pair_index[1]; index += 2){
			
			i = first_in_cell[cell_pair[index]];
			jhead = first_in_cell[cell_pair[index+1]];
			while (i != -1){

				ki = m_knd[i];
				//nabor loop ----;
				j = jhead;
				while (j != -1){
					dd = m_r[i] - m_r[j];
					l += AddBondByCutOff(i,j,ki,m_knd[j],l,dd);	//bondǉꂽ1Ԃ.

					j = inext[j];
				}// end of nabor loop;
	
				i = inext[i];
			}
		}

		//xEׂ(jxzcell)
			peri = - Am.a;
			for( index = info->cell_pair_index[1]; index < info->cell_pair_index[2]; index += 2){
				
				
				i = first_in_cell[cell_pair[index]];
				jhead = first_in_cell[cell_pair[index+1]];
				while (i != -1){

					ki = m_knd[i];
					//nabor loop ----;
					j = jhead;
					while (j != -1){
						dd = m_r[i] - m_r[j] + peri;
						l += AddBondByCutOff(i,j,ki,m_knd[j],l,dd);	//bondǉꂽ1Ԃ.

						j = inext[j];
					}// end of nabor loop;
		
					i = inext[i];
				}
			}
		
		//yEׂ(jyzcell)
			peri = - Am.b;
			for( index = info->cell_pair_index[2]; index < info->cell_pair_index[3]; index += 2){
				
				
				i = first_in_cell[cell_pair[index]];
				jhead = first_in_cell[cell_pair[index+1]];
				while (i != -1){

					ki = m_knd[i];
					//nabor loop ----;
					j = jhead;
					while (j != -1){
						//GetRij_periodicY(i,j, &xx, &yy, &zz);
						dd = m_r[i] - m_r[j] + peri;
						l += AddBondByCutOff(i,j,ki,m_knd[j],l,dd);	//bondǉꂽ1Ԃ.

						j = inext[j];
					}// end of nabor loop;
		
					i = inext[i];
				}
			}

		//zEׂ(jzzcell)
			peri = - Am.c;
			for( index = info->cell_pair_index[3]; index < info->cell_pair_index[4]; index += 2){
				
				
				i = first_in_cell[cell_pair[index]];
				jhead = first_in_cell[cell_pair[index+1]];
				while (i != -1){

					ki = m_knd[i];
					//nabor loop ----;
					j = jhead;
					while (j != -1){
						//GetRij_periodicZ(i,j, &xx, &yy, &zz);
						dd = m_r[i] - m_r[j] + peri;
						l += AddBondByCutOff(i,j,ki,m_knd[j],l,dd);	//bondǉꂽ1Ԃ.

						j = inext[j];
					}// end of nabor loop;
		
					i = inext[i];
				}
			}

		//xyEׂ(҂xyɉzcell)
			peri = - Am.a - Am.b;
			for( index = info->cell_pair_index[4]; index < info->cell_pair_index[5]; index += 2){
				
				i = first_in_cell[cell_pair[index]];
				jhead = first_in_cell[cell_pair[index+1]];
				while (i != -1){

					ki = m_knd[i];
					//nabor loop ----;
					j = jhead;
					while (j != -1){
						//GetRij_periodicXY(i,j, &xx, &yy, &zz);
						//l += AddBondByCutOff(i,j,ki,l,xx,yy,zz);	//bondǉꂽ1Ԃ.
						dd = m_r[i] - m_r[j] + peri;
						l += AddBondByCutOff(i,j,ki,m_knd[j],l,dd);	//bondǉꂽ1Ԃ.

						j = inext[j];
					}// end of nabor loop;
		
					i = inext[i];
				}
			}
		//xyEׂ(O҂xzcell,҂yzcell)
			peri = Am.a - Am.b;
			for( index = info->cell_pair_index[5]; index < info->cell_pair_index[6]; index += 2){
				
				i = first_in_cell[cell_pair[index]];
				jhead = first_in_cell[cell_pair[index+1]];
				while (i != -1){

					ki = m_knd[i];
					//nabor loop ----;
					j = jhead;
					while (j != -1){
						//GetRij_periodicXY(i,j, &xx, &yy, &zz);
						//l += AddBondByCutOff(i,j,ki,l,xx,yy,zz);	//bondǉꂽ1Ԃ.
						dd = m_r[i] - m_r[j] + peri;
						l += AddBondByCutOff(i,j,ki,m_knd[j],l,dd);	//bondǉꂽ1Ԃ.

						j = inext[j];
					}// end of nabor loop;
		
					i = inext[i];
				}
			}

		//yzEׂ(҂yzɉzcell)
			peri = - Am.b - Am.c;
			for( index = info->cell_pair_index[6]; index < info->cell_pair_index[7]; index += 2){
				
				i = first_in_cell[cell_pair[index]];
				jhead = first_in_cell[cell_pair[index+1]];
				while (i != -1){

					ki = m_knd[i];
					//nabor loop ----;
					j = jhead;
					while (j != -1){
						//GetRij_periodicYZ(i,j, &xx, &yy, &zz);
						//l += AddBondByCutOff(i,j,ki,l,xx,yy,zz);	//bondǉꂽ1Ԃ.
						dd = m_r[i] - m_r[j] + peri;
						l += AddBondByCutOff(i,j,ki,m_knd[j],l,dd);	//bondǉꂽ1Ԃ.

						j = inext[j];
					}// end of nabor loop;
		
					i = inext[i];
				}
			}
		//yzEׂ(O҂yzcell,҂zzcell)
			peri = Am.b - Am.c;
			for( index = info->cell_pair_index[7]; index < info->cell_pair_index[8]; index += 2){
				
				i = first_in_cell[cell_pair[index]];
				jhead = first_in_cell[cell_pair[index+1]];
				while (i != -1){

					ki = m_knd[i];
					//nabor loop ----;
					j = jhead;
					while (j != -1){
						//GetRij_periodicYZ(i,j, &xx, &yy, &zz);
						//l += AddBondByCutOff(i,j,ki,l,xx,yy,zz);	//bondǉꂽ1Ԃ.
						dd = m_r[i] - m_r[j] + peri;
						l += AddBondByCutOff(i,j,ki,m_knd[j],l,dd);	//bondǉꂽ1Ԃ.

						j = inext[j];
					}// end of nabor loop;
		
					i = inext[i];
				}
			}
			
			
		//zxEׂ(҂zxɉzcell)
			peri = - Am.a - Am.c;
			for( index = info->cell_pair_index[8]; index < info->cell_pair_index[9]; index += 2){
				
				i = first_in_cell[cell_pair[index]];
				jhead = first_in_cell[cell_pair[index+1]];
				while (i != -1){

					ki = m_knd[i];
					//nabor loop ----;
					j = jhead;
					while (j != -1){
						//GetRij_periodicZX(i,j, &xx, &yy, &zz);
						//l += AddBondByCutOff(i,j,ki,l,xx,yy,zz);	//bondǉꂽ1Ԃ.
						dd = m_r[i] - m_r[j] + peri;
						l += AddBondByCutOff(i,j,ki,m_knd[j],l,dd);	//bondǉꂽ1Ԃ.

						j = inext[j];
					}// end of nabor loop;
		
					i = inext[i];
				}
			}
		//zxEׂ(O҂zzcell,҂xzcell)
			peri = - Am.a + Am.c;
			for( index = info->cell_pair_index[9]; index < info->cell_pair_index[10]; index += 2){
				
				i = first_in_cell[cell_pair[index]];
				jhead = first_in_cell[cell_pair[index+1]];
				while (i != -1){

					ki = m_knd[i];
					//nabor loop ----;
					j = jhead;
					while (j != -1){
						//GetRij_periodicZX(i,j, &xx, &yy, &zz);
						//l += AddBondByCutOff(i,j,ki,l,xx,yy,zz);	//bondǉꂽ1Ԃ.
						dd = m_r[i] - m_r[j] + peri;
						l += AddBondByCutOff(i,j,ki,m_knd[j],l,dd);	//bondǉꂽ1Ԃ.

						j = inext[j];
					}// end of nabor loop;
		
					i = inext[i];
				}
			}

		//xyzEׂ(O҂xzcell,҂yzzcell)
			peri = Am.a - Am.b - Am.c;
			index = info->cell_pair_index[10];
				
				i = first_in_cell[cell_pair[index]];
				jhead = first_in_cell[cell_pair[index+1]];
				while (i != -1){

					ki = m_knd[i];
					//nabor loop ----;
					j = jhead;
					while (j != -1){
						//GetRij(i,j, &xx, &yy, &zz);
						//l += AddBondByCutOff(i,j,ki,l,xx,yy,zz);	//bondǉꂽ1Ԃ.
						dd = m_r[i] - m_r[j] + peri;
						l += AddBondByCutOff(i,j,ki,m_knd[j],l,dd);	//bondǉꂽ1Ԃ.

						j = inext[j];
					}// end of nabor loop;
		
					i = inext[i];
				}
			
			//(O҂yzcell,҂zxzcell)
			peri = - Am.a + Am.b - Am.c;
			index +=2;
			
				i = first_in_cell[cell_pair[index]];
				jhead = first_in_cell[cell_pair[index+1]];
				while (i != -1){

					ki = m_knd[i];
					//nabor loop ----;
					j = jhead;
					while (j != -1){
						//GetRij(i,j, &xx, &yy, &zz);
						//l += AddBondByCutOff(i,j,ki,l,xx,yy,zz);	//bondǉꂽ1Ԃ.
						dd = m_r[i] - m_r[j] + peri;
						l += AddBondByCutOff(i,j,ki,m_knd[j],l,dd);	//bondǉꂽ1Ԃ.

						j = inext[j];
					}// end of nabor loop;
		
					i = inext[i];
				}
			//(O҂zzcell,҂xyzcell)
			peri = - Am.a - Am.b + Am.c;
			index +=2;
			
				i = first_in_cell[cell_pair[index]];
				jhead = first_in_cell[cell_pair[index+1]];
				while (i != -1){

					ki = m_knd[i];
					//nabor loop ----;
					j = jhead;
					while (j != -1){
						//GetRij(i,j, &xx, &yy, &zz);
						//l += AddBondByCutOff(i,j,ki,l,xx,yy,zz);	//bondǉꂽ1Ԃ.
						dd = m_r[i] - m_r[j] + peri;
						l += AddBondByCutOff(i,j,ki,m_knd[j],l,dd);	//bondǉꂽ1Ԃ.

						j = inext[j];
					}// end of nabor loop;
		
					i = inext[i];
				}
			
			//(҂xyzzcell)
			peri = - Am.a - Am.b - Am.c;
			index +=2;
			
				i = first_in_cell[cell_pair[index]];
				jhead = first_in_cell[cell_pair[index+1]];
				while (i != -1){

					ki = m_knd[i];
					//nabor loop ----;
					j = jhead;
					while (j != -1){
						//GetRij(i,j, &xx, &yy, &zz);
						//l += AddBondByCutOff(i,j,ki,l,xx,yy,zz);	//bondǉꂽ1Ԃ.
						dd = m_r[i] - m_r[j] + peri;
						l += AddBondByCutOff(i,j,ki,m_knd[j],l,dd);	//bondǉꂽ1Ԃ.

						j = inext[j];
					}// end of nabor loop;
		
					i = inext[i];
				}
			


//		}

		m_bondCnt = l;

		
}

/*
void REBO::SetListNonortho_v20101124(mat33d Am){
	
	// local variable ------------------------------;
		int i,j,l, jhead;
		//double xx,yy,zz;
		int ki;
		vec3d dd;
		vec3d peri;


		l = 0;


	//Zł̒T==============================================
		int icell;
		for( icell=0; icell < g_mcell; icell++){
				//loop over all molecules in the cell;
					i = g_ihead[icell];
					while (i != -1){

						ki = m_knd[i];
						j = g_inxt[i];
		
				
						//nabor loop ----;
						while (j != -1){

							dd = m_r[i] - m_r[j];
							l += AddBondByCutOff(i,j,ki,l,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.

							j = g_inxt[j];
						}
						
						i = g_inxt[i];		
					}

		}

	//ʂ̃ZԂł̒T==============================================
		//E1cell.
		//Eɂ܂肽݂̔B
		int index;
		for( index=0; index < g_cell_index[1]; index += 2){
			
			i = g_ihead[g_map_pair[index]];
			jhead = g_ihead[g_map_pair[index+1]];
			while (i != -1){

				ki = m_knd[i];
				//nabor loop ----;
				j = jhead;
				while (j != -1){
					dd = m_r[i] - m_r[j];
					l += AddBondByCutOff(i,j,ki,l,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
					j = g_inxt[j];
				}// end of nabor loop;
	
				i = g_inxt[i];
			}
		}

		//xEׂ(jxzcell)
			peri = - Am.a;
			for( index = g_cell_index[1]; index < g_cell_index[2]; index += 2){
				
				
				i = g_ihead[g_map_pair[index]];
				jhead = g_ihead[g_map_pair[index+1]];
				while (i != -1){

					ki = m_knd[i];
					//nabor loop ----;
					j = jhead;
					while (j != -1){
						dd = m_r[i] - m_r[j] + peri;
						l += AddBondByCutOff(i,j,ki,l,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
						j = g_inxt[j];
					}// end of nabor loop;
		
					i = g_inxt[i];
				}
			}
		
		//yEׂ(jyzcell)
			peri = - Am.b;
			for( index = g_cell_index[2]; index < g_cell_index[3]; index += 2){
				
				
				i = g_ihead[g_map_pair[index]];
				jhead = g_ihead[g_map_pair[index+1]];
				while (i != -1){

					ki = m_knd[i];
					//nabor loop ----;
					j = jhead;
					while (j != -1){
						//GetRij_periodicY(i,j, &xx, &yy, &zz);
						dd = m_r[i] - m_r[j] + peri;
						l += AddBondByCutOff(i,j,ki,l,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
						j = g_inxt[j];
					}// end of nabor loop;
		
					i = g_inxt[i];
				}
			}

		//zEׂ(jzzcell)
			peri = - Am.c;
			for( index = g_cell_index[3]; index < g_cell_index[4]; index += 2){
				
				
				i = g_ihead[g_map_pair[index]];
				jhead = g_ihead[g_map_pair[index+1]];
				while (i != -1){

					ki = m_knd[i];
					//nabor loop ----;
					j = jhead;
					while (j != -1){
						//GetRij_periodicZ(i,j, &xx, &yy, &zz);
						dd = m_r[i] - m_r[j] + peri;
						l += AddBondByCutOff(i,j,ki,l,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
						j = g_inxt[j];
					}// end of nabor loop;
		
					i = g_inxt[i];
				}
			}

		//xyEׂ(҂xyɉzcell)
			peri = - Am.a - Am.b;
			for( index = g_cell_index[4]; index < g_cell_index[5]; index += 2){
				
				i = g_ihead[g_map_pair[index]];
				jhead = g_ihead[g_map_pair[index+1]];
				while (i != -1){

					ki = m_knd[i];
					//nabor loop ----;
					j = jhead;
					while (j != -1){
						//GetRij_periodicXY(i,j, &xx, &yy, &zz);
						//l += AddBondByCutOff(i,j,ki,l,xx,yy,zz);	//bondǉꂽ1Ԃ.
						dd = m_r[i] - m_r[j] + peri;
						l += AddBondByCutOff(i,j,ki,l,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
						j = g_inxt[j];
					}// end of nabor loop;
		
					i = g_inxt[i];
				}
			}
		//xyEׂ(O҂xzcell,҂yzcell)
			peri = Am.a - Am.b;
			for( index = g_cell_index[5]; index < g_cell_index[6]; index += 2){
				
				i = g_ihead[g_map_pair[index]];
				jhead = g_ihead[g_map_pair[index+1]];
				while (i != -1){

					ki = m_knd[i];
					//nabor loop ----;
					j = jhead;
					while (j != -1){
						//GetRij_periodicXY(i,j, &xx, &yy, &zz);
						//l += AddBondByCutOff(i,j,ki,l,xx,yy,zz);	//bondǉꂽ1Ԃ.
						dd = m_r[i] - m_r[j] + peri;
						l += AddBondByCutOff(i,j,ki,l,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
						j = g_inxt[j];
					}// end of nabor loop;
		
					i = g_inxt[i];
				}
			}

		//yzEׂ(҂yzɉzcell)
			peri = - Am.b - Am.c;
			for( index = g_cell_index[6]; index < g_cell_index[7]; index += 2){
				
				i = g_ihead[g_map_pair[index]];
				jhead = g_ihead[g_map_pair[index+1]];
				while (i != -1){

					ki = m_knd[i];
					//nabor loop ----;
					j = jhead;
					while (j != -1){
						//GetRij_periodicYZ(i,j, &xx, &yy, &zz);
						//l += AddBondByCutOff(i,j,ki,l,xx,yy,zz);	//bondǉꂽ1Ԃ.
						dd = m_r[i] - m_r[j] + peri;
						l += AddBondByCutOff(i,j,ki,l,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
						j = g_inxt[j];
					}// end of nabor loop;
		
					i = g_inxt[i];
				}
			}
		//yzEׂ(O҂yzcell,҂zzcell)
			peri = Am.b - Am.c;
			for( index = g_cell_index[7]; index < g_cell_index[8]; index += 2){
				
				i = g_ihead[g_map_pair[index]];
				jhead = g_ihead[g_map_pair[index+1]];
				while (i != -1){

					ki = m_knd[i];
					//nabor loop ----;
					j = jhead;
					while (j != -1){
						//GetRij_periodicYZ(i,j, &xx, &yy, &zz);
						//l += AddBondByCutOff(i,j,ki,l,xx,yy,zz);	//bondǉꂽ1Ԃ.
						dd = m_r[i] - m_r[j] + peri;
						l += AddBondByCutOff(i,j,ki,l,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
						j = g_inxt[j];
					}// end of nabor loop;
		
					i = g_inxt[i];
				}
			}
			
			
		//zxEׂ(҂zxɉzcell)
			peri = - Am.a - Am.c;
			for( index = g_cell_index[8]; index < g_cell_index[9]; index += 2){
				
				i = g_ihead[g_map_pair[index]];
				jhead = g_ihead[g_map_pair[index+1]];
				while (i != -1){

					ki = m_knd[i];
					//nabor loop ----;
					j = jhead;
					while (j != -1){
						//GetRij_periodicZX(i,j, &xx, &yy, &zz);
						//l += AddBondByCutOff(i,j,ki,l,xx,yy,zz);	//bondǉꂽ1Ԃ.
						dd = m_r[i] - m_r[j] + peri;
						l += AddBondByCutOff(i,j,ki,l,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
						j = g_inxt[j];
					}// end of nabor loop;
		
					i = g_inxt[i];
				}
			}
		//zxEׂ(O҂zzcell,҂xzcell)
			peri = - Am.a + Am.c;
			for( index = g_cell_index[9]; index < g_cell_index[10]; index += 2){
				
				i = g_ihead[g_map_pair[index]];
				jhead = g_ihead[g_map_pair[index+1]];
				while (i != -1){

					ki = m_knd[i];
					//nabor loop ----;
					j = jhead;
					while (j != -1){
						//GetRij_periodicZX(i,j, &xx, &yy, &zz);
						//l += AddBondByCutOff(i,j,ki,l,xx,yy,zz);	//bondǉꂽ1Ԃ.
						dd = m_r[i] - m_r[j] + peri;
						l += AddBondByCutOff(i,j,ki,l,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
						j = g_inxt[j];
					}// end of nabor loop;
		
					i = g_inxt[i];
				}
			}

		//xyzEׂ(O҂xzcell,҂yzzcell)
			peri = Am.a - Am.b - Am.c;
			index = g_cell_index[10];
				
				i = g_ihead[g_map_pair[index]];
				jhead = g_ihead[g_map_pair[index+1]];
				while (i != -1){

					ki = m_knd[i];
					//nabor loop ----;
					j = jhead;
					while (j != -1){
						//GetRij(i,j, &xx, &yy, &zz);
						//l += AddBondByCutOff(i,j,ki,l,xx,yy,zz);	//bondǉꂽ1Ԃ.
						dd = m_r[i] - m_r[j] + peri;
						l += AddBondByCutOff(i,j,ki,l,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
						j = g_inxt[j];
					}// end of nabor loop;
		
					i = g_inxt[i];
				}
			
			//(O҂yzcell,҂zxzcell)
			peri = - Am.a + Am.b - Am.c;
			index +=2;
			
				i = g_ihead[g_map_pair[index]];
				jhead = g_ihead[g_map_pair[index+1]];
				while (i != -1){

					ki = m_knd[i];
					//nabor loop ----;
					j = jhead;
					while (j != -1){
						//GetRij(i,j, &xx, &yy, &zz);
						//l += AddBondByCutOff(i,j,ki,l,xx,yy,zz);	//bondǉꂽ1Ԃ.
						dd = m_r[i] - m_r[j] + peri;
						l += AddBondByCutOff(i,j,ki,l,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
						j = g_inxt[j];
					}// end of nabor loop;
		
					i = g_inxt[i];
				}
			//(O҂zzcell,҂xyzcell)
			peri = - Am.a - Am.b + Am.c;
			index +=2;
			
				i = g_ihead[g_map_pair[index]];
				jhead = g_ihead[g_map_pair[index+1]];
				while (i != -1){

					ki = m_knd[i];
					//nabor loop ----;
					j = jhead;
					while (j != -1){
						//GetRij(i,j, &xx, &yy, &zz);
						//l += AddBondByCutOff(i,j,ki,l,xx,yy,zz);	//bondǉꂽ1Ԃ.
						dd = m_r[i] - m_r[j] + peri;
						l += AddBondByCutOff(i,j,ki,l,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
						j = g_inxt[j];
					}// end of nabor loop;
		
					i = g_inxt[i];
				}
			
			//(҂xyzzcell)
			peri = - Am.a - Am.b - Am.c;
			index +=2;
			
				i = g_ihead[g_map_pair[index]];
				jhead = g_ihead[g_map_pair[index+1]];
				while (i != -1){

					ki = m_knd[i];
					//nabor loop ----;
					j = jhead;
					while (j != -1){
						//GetRij(i,j, &xx, &yy, &zz);
						//l += AddBondByCutOff(i,j,ki,l,xx,yy,zz);	//bondǉꂽ1Ԃ.
						dd = m_r[i] - m_r[j] + peri;
						l += AddBondByCutOff(i,j,ki,l,dd.x,dd.y,dd.z);	//bondǉꂽ1Ԃ.
						j = g_inxt[j];
					}// end of nabor loop;
		
					i = g_inxt[i];
				}
			


//		}

		m_bondCnt = l;

		
}
*/
/*
int REBO::AddBondByCutOff(int i, int j, int ki, int Lidx, double xx, double yy, double zz){
	double rd2,rd,ird;
	double r3, r2, r;
	
	int iprt,jprt;
	
	double fco, dfco;

	
	rd2 = xx * xx + yy * yy + zz * zz;

	int kj = m_knd[j];

	//if (rd2 <= m_Dmax[ki + kj]*m_Dmax[ki + kj]) {
	if (rd2 <= m_Dmax2[ki + kj]) {
		rd = sqrt(rd2);		////new;

		if (rd <= m_Dmin[ki + kj]) {
			fco = 1.0;
			dfco = 0.0;
		}else{
			
		//	Res = PI * ( rd - m_Dmin[ki + kj]) / (m_Dmax[ki + kj] - m_Dmin[ki + kj]);
		//	fco = 0.5 * (1.0 + cos(Res));
		//	dfco = - 0.5 * PI / (m_Dmax[ki + kj] - m_Dmin[ki + kj]) * sin(Res);

			r = ( rd - m_Dmin[ki + kj]) / (m_Ddiff[ki + kj]);
			r2 = r * r;
			r3 = r2 * r;
			fco = (- 6.0 * r2 + 15.0 * r - 10.0) * r3 + 1.0;
			dfco = - 30.0 * (r - 1.0) * (r - 1.0) * r2 / (m_Ddiff[ki + kj]);
		

		}

		REBO_PARTICLE* rp_i = m_rp + i;
		REBO_PARTICLE* rp_j = m_rp + j;
		
		if(kj==KIND_C){
			rp_i->Nc += fco;		////g_fc(Lidx);
		}else{
			rp_i->Nh += fco;		////g_fc(Lidx);
		}
		
		if(ki==KIND_C){
			rp_j->Nc += fco;		////g_fc(Lidx);
		}else{
			rp_j->Nh += fco;		////g_fc(Lidx);
		}
		
		
		iprt = rp_i->NbCnt;
		rp_i->NbCnt ++;
		
		
		jprt = rp_j->NbCnt;
		rp_j->NbCnt ++;


//ǂNI[o[t[̌
		if(rp_i->NbCnt >= 12){
			rp_i->NbCnt = 0;
		}
		if(rp_j->NbCnt >= 12){
			rp_j->NbCnt = 0;
		}
	
		m_bonding[Lidx].i = i;
		m_bonding[Lidx].bondnum_i = iprt;
		m_bonding[Lidx].j = j;
		m_bonding[Lidx].bondnum_j = jprt;
		
		rp_i->NbPrt[iprt] = j;
		rp_j->NbPrt[jprt] = i;

		ird = 1.0 / rd;

		rp_i->fc[iprt] = fco;
		rp_i->dfc[iprt] = dfco;
		rp_i->dr[iprt] = rd;
		rp_i->idr[iprt] = ird;
		

		rp_j->fc[jprt] = fco;
		rp_j->dfc[jprt] = dfco;
		rp_j->dr[jprt] = rd;
		rp_j->idr[jprt] = ird;
		
	//////unit vector;
		
		xx = xx * ird;
		yy = yy * ird;
		zz = zz * ird;
		rp_i->e[iprt].Set(xx, yy, zz);
		rp_j->e[jprt].Set(-xx, -yy, -zz);

		return 1;
	}
	return 0;
}
*/

inline int REBO::AddBondByCutOff(int i, int j, int ki, int kj, int Lidx, vec3d& dd){
	double rd2,rd,ird;
	double r3, r2, r;
	
	int iprt,jprt;
	
	double fco, dfco;

	
	rd2 = dd * dd;

	//int kj = m_knd[j];

	//if (rd2 <= m_Dmax[ki + kj]*m_Dmax[ki + kj]) {
	if (rd2 <= m_Dmax2[ki + kj]) {
		rd = sqrt(rd2);		////new;

		if (rd <= m_Dmin[ki + kj]) {
			fco = 1.0;
			dfco = 0.0;
		}else{
			/*
			Res = PI * ( rd - m_Dmin[ki + kj]) / (m_Dmax[ki + kj] - m_Dmin[ki + kj]);
			fco = 0.5 * (1.0 + cos(Res));
			dfco = - 0.5 * PI / (m_Dmax[ki + kj] - m_Dmin[ki + kj]) * sin(Res);*/

			r = ( rd - m_Dmin[ki + kj]) / (m_Ddiff[ki + kj]);
			r2 = r * r;
			r3 = r2 * r;
			fco = (- 6.0 * r2 + 15.0 * r - 10.0) * r3 + 1.0;
			dfco = - 30.0 * (r - 1.0) * (r - 1.0) * r2 / (m_Ddiff[ki + kj]);
		

		}

		REBO_PARTICLE* rp_i = m_rp + i;
		REBO_PARTICLE* rp_j = m_rp + j;
		
		rp_i->Nx[kj] += fco;	
		rp_j->Nx[ki] += fco;
		
		iprt = rp_i->NbCnt;
		rp_i->NbCnt ++;		
		if(rp_i->NbCnt >= BOND_CNT){	//ǂNI[o[t[̌
			rp_i->NbCnt = 0;
		}
		
		jprt = rp_j->NbCnt;
		rp_j->NbCnt ++;
		if(rp_j->NbCnt >= BOND_CNT){	//ǂNI[o[t[̌
			rp_j->NbCnt = 0;
		}
	
		m_bonding[Lidx].i = i;
		m_bonding[Lidx].bondnum_i = iprt;
		m_bonding[Lidx].j = j;
		m_bonding[Lidx].bondnum_j = jprt;
		
		rp_i->NbPrt[iprt] = j;
		rp_j->NbPrt[jprt] = i;

		ird = 1.0 / rd;

		rp_i->fc[iprt] = fco;
		rp_i->dfc[iprt] = dfco;
		rp_i->dr[iprt] = rd;
		rp_i->idr[iprt] = ird;
		

		rp_j->fc[jprt] = fco;
		rp_j->dfc[jprt] = dfco;
		rp_j->dr[jprt] = rd;
		rp_j->idr[jprt] = ird;
		
	//////unit vector;
		
		dd *= ird;
		rp_i->e[iprt] = dd;
		rp_j->e[jprt] = -dd;
		return 1;
	}
	return 0;
}

//======================================================================;
//;
//			subroutine ForceBrenner;
//;
//				this routine calculate force for brenner potential;
//;
//;
//;
//									made by Atsushi I.	2004/12/20;
//									last modification  2005/01/13;
//======================================================================;

void REBO::Force(int flag){

		

	//local ----------------------------------------;
#define	EPS		(1.0e-8)
		double rij,idr;
//		double xx,yy,zz;
		//double ex,ey,ez;
		vec3d exyz;
		
		int i,j,iprt,ki,kj,kndbros,Lidx,jprt;
		double Bvar, B_ij, B_ji, PI_RC, B_DH;
		double Nc_i, Nh_i,Nc_j, Nh_j, Nt_ij, Nt_ji, Nconj;
		double spP;
	
		//for force;
		double V_A1, V_A2, V_A3, V_A,V_R , V2;
		double FrcA, coef,FrcR;  ////, t_Epot;
		
		
		double cosI[BOND_CNT];
		double cosJ[BOND_CNT];
		vec3d cosI_dIK[BOND_CNT], cosI_dIJ[BOND_CNT];
		vec3d cosJ_dJL[BOND_CNT], cosJ_dJI[BOND_CNT];
		
		double Gsum_i,Gsum2_i;
		double Gary_i[BOND_CNT],Gdifary_i[BOND_CNT];
		double Gsum_j,Gsum2_j;
		double Gary_j[BOND_CNT],Gdifary_j[BOND_CNT];
		
		double cosDH[BOND_CNT*BOND_CNT];
		double T_ij,cosDH2sum;
		
		double Ssum_i,Ssum_j;
		double S_ki[BOND_CNT],dS_ki[BOND_CNT];
		double S_lj[BOND_CNT],dS_lj[BOND_CNT];
		
		double fco,dfco;

		//test
		//double ftes(0:2),res,res2;	//,SplinePdx,SplinePdy;
		
		
//		int interactionCnt;
	//coefficient for brenner potential;
		static const double B1[3] = {1.238879197798e4, 32.3551866587, 29.632593};
		static const double B2[3] = {17.56740646509, 0.0, 0.0};
		static const double B3[3] = {3.071493208065e1, 0.0, 0.0};
		static const double beta1[3] = {4.7204523127, 1.43445805925, 1.71589217};
		static const double beta2[3] = {1.4332132499, 0.0, 0.0};
		static const double beta3[3] = {1.3826912506, 0.0, 0.0};
		static const double Q[3] = {0.3134602960833, 0.340775728, 0.370471487045};
		static const double A[3] = {1.0953544162170e4, 149.94098723, 32.817355747};
		static const double alpha[3] = {4.7465390606595, 4.10254983, 3.536298648};
		

		
		
		
		for (Lidx=0; Lidx < m_bondCnt; Lidx++){
			
			i = m_bonding[Lidx].i;
			j = m_bonding[Lidx].j;
		
			ki = m_knd[i];
			kj = m_knd[j];
			kndbros = ki + kj;
			
			iprt = m_bonding[Lidx].bondnum_i;
			jprt = m_bonding[Lidx].bondnum_j;
	
			
		
			
			fco = m_rp[i].fc[iprt];
			dfco = m_rp[i].dfc[iprt];
			
			rij = m_rp[i].dr[iprt];
			idr = m_rp[i].idr[iprt];
			exyz = m_rp[i].e[iprt];
			

			CalcCos3_4(i,j,exyz,idr,cosI, cosI_dIK, cosI_dIJ);
			CalcCos3_4(j,i,-exyz,idr,cosJ, cosJ_dJL, cosJ_dJI);
			

			CalcFor3Body(i,j,cosI,&Gsum_i,Gary_i,Gdifary_i,&Gsum2_i);
			CalcFor3Body(j,i,cosJ,&Gsum_j,Gary_j,Gdifary_j,&Gsum2_j);

			
			B_ij = 1.0 + Gsum_i;
			B_ji = 1.0 + Gsum_j;
			
			
			if(kj==KIND_C){
				Nc_i = m_rp[i].Nc - fco;		////g_fc(Lidx);
				Nh_i = m_rp[i].Nh;
			}else{
				Nc_i = m_rp[i].Nc;
				Nh_i = m_rp[i].Nh - fco;		////g_fc(Lidx);
			}
			
			if(ki==KIND_C){
				Nc_j = m_rp[j].Nc - fco;		////g_fc(Lidx);
				Nh_j = m_rp[j].Nh;
			}else{
				Nc_j = m_rp[j].Nc;
				Nh_j = m_rp[j].Nh - fco;		////g_fc(Lidx);
			}
			
			spP = 0.0;
		//term of function P(Nh_i,Nc_i);
			if (ki == KIND_C) {
				spP = SplineP_2(Nh_i, Nc_i,kj);
				B_ij += spP;
			}
			
			if (kj == KIND_C) {
				spP = SplineP_2(Nh_j, Nc_j, ki);
				B_ji += spP;
			}
			
		//finish Bij;
			B_ij = 1.0 / sqrt(B_ij);
			B_ji = 1.0 / sqrt(B_ji);
		


		//calculate conjugation;
			CalcForConjugation(i,j,&Ssum_i,S_ki,dS_ki);
			CalcForConjugation(j,i,&Ssum_j,S_lj,dS_lj);
			

			Nconj = 1.0 + Ssum_i + Ssum_j;
			
			Nt_ij = m_rp[i].Nc + m_rp[i].Nh - fco;		////g_fc(Lidx);
			Nt_ji = m_rp[j].Nc + m_rp[j].Nh - fco;		////g_fc(Lidx);

			
			PI_RC = SplineF_2(Nt_ij,Nt_ji,Nconj,ki + kj);
			

			CalcFor4Body(i,j,cosI,cosJ,&cosDH2sum,cosDH);
			if (ki + kj == KIND_CC) {
				T_ij = SplineT_2(Nt_ij,Nt_ji,Nconj);
				B_DH = T_ij * cosDH2sum;
			}else{
				T_ij = 0.0;
				B_DH = 0.0;
				cosDH2sum = 0.0;
			}
			


			Bvar = 0.5 * (B_ij + B_ji) + PI_RC + B_DH;
	//calculate coefficient end =============================;
			

			V_A1 = B1[kndbros] * exp( -beta1[kndbros] * rij);
			V_A2 = B2[kndbros] * exp( -beta2[kndbros] * rij);
			V_A3 = B3[kndbros] * exp( -beta3[kndbros] * rij);
			
//--------------force by two bodys (bond)----------------------;
			FrcA = beta1[kndbros] * V_A1;
			FrcA = FrcA + beta2[kndbros] * V_A2;
			FrcA = FrcA + beta3[kndbros] * V_A3;
			FrcA =  - fco * FrcA + dfco*(V_A1 + V_A2 + V_A3);

			V_A = (V_A1 + V_A2 + V_A3) * fco;		////g_fc(Lidx);
			
	/////////!====repulsion;
			V_R = A[kndbros] * exp( - alpha[kndbros] * rij );
			FrcR = alpha[kndbros] * (1 + Q[kndbros] * idr) + Q[kndbros] * (idr*idr);
			FrcR = FrcR * fco - dfco * (1 + Q[kndbros] * idr);
			FrcR = FrcR * V_R;
			
			V2 = (1.0 + Q[kndbros] * idr) * V_R * fco - Bvar * V_A;
			
			m_pot[i] += V2;
			m_pot[j] += V2;

//			g_REBO_Pot = g_REBO_Pot + V2;
	/////////!===============repulsion_fin====;

			coef = Bvar * FrcA + FrcR;

			//frc = coef * ex;

			m_rp[i].fij[iprt] += exyz * coef;

	//---------- force by three body (Bond angles)-------------------;

			Force3Body_2(i,j,V_A,B_ij,cosI_dIK,cosI_dIJ,Gary_i,Gdifary_i,Gsum2_i,iprt);
			Force3Body_2(j,i,V_A,B_ji,cosJ_dJL,cosJ_dJI,Gary_j,Gdifary_j,Gsum2_j,jprt);

			ForceSplineP(i,j,V_A,B_ij,Nh_i,Nc_i);
			ForceSplineP(j,i,V_A,B_ji,Nh_j,Nc_j);

	//---------- force by pi-bond -------------------;
			ForcePiBond(i,j,V_A,Nt_ij,Nt_ji,Nconj,Ssum_i,S_ki,dS_ki,Ssum_j,S_lj,dS_lj);


	//---------- force by four body (Dihedral angles)-------------------;
			Force4Body_2(i,j,V_A,T_ij,cosI,cosI_dIK,cosI_dIJ,cosJ,cosJ_dJL,cosJ_dJI,cosDH,iprt);

			ForceSplineT(i,j,V_A,Nt_ij,Nt_ji,Nconj,cosDH2sum,Ssum_i,S_ki,dS_ki,Ssum_j,S_lj,dS_lj);
			

//---------- potential -------------------;

		}
		
		switch (flag){
		case 1:
			CalcFJ();
			break;
		case 2:
			CalcFJFrv();
			break;
		case 3:
			CalcFJFrm();
			break;
		default:
			CalcF();
		}
	
		
		
}

//==================================================================;
//;
//		function subG;
//;
//			calculating spline function G;
//;
//									made by Atsushi I.	2005/06/29;
//==================================================================;



inline void REBO::subG(double cosVal, int mode, double *retVal){
		

		double res;
		double x,x2,x3,x4,x5,x6;
		double h0, h1, h2, h3, h4, h5, h6;
		int k;
		const double cos0 = 1.0;
		const double cos109 = -0.333411977216052;			//dcos(6.082rad);
		const double cos120 = -0.5;
		const double cos180 = -1.0;
		
		const double Gcoeff[6][4] = {
			{-0.001, 0.10400, 0.0, 0.0},
			{0.05280, 0.17, 0.37, -5.232},
			{0.09733, 0.40, 1.98, 41.6140},
			{8.0, 0.23622, -166.1360, 0.0},
			{0.09733, 0.40, 1.98, -9.95630270},
			{1.0, 0.78, -11.3022275, 0.0}};
		
		double delta;
		
		if (cosVal<=cos120){
			delta = cos120-cos180;
			x = (cosVal - cos180)/delta;
			k = 0;
		}else if (cosVal<=cos109) {
			delta = cos109-cos120;
			x = (cosVal - cos120)/delta;
			k = 1;
		}else if (mode == 1) {
			delta = cos0 - cos109;
			x = (cosVal - cos109)/delta;
			k = 2;
		}else if (mode == 2) {
			delta = cos0 - cos109;
			x = (cosVal - cos109)/delta;
			k = 4;
		}else{
			TRACE(_T("error spline G\n"));
		}
		
		
		x2 = x*x;
		x3 = x2*x;
		x4 = x2 * x2;
		x5 = x4 * x;
		x6 = x4 * x2;

		h0 = 1.0 - 15.0*x4 + 24.0*x5 - 10.0*x6;
		h1 = 15.0*x4 - 24.0*x5 + 10.0*x6;
		h2 = x - 10.0*x4 + 15.0*x5 - 6.0*x6;
		h3 = - 5.0*x4 + 9.0*x5 - 4.0*x6;
		h4 = 0.5*x2 - 3.0*x4 +4.0*x5 - 1.5*x6;
		h5 = 0.5*x4 - x5 + 0.5*x6;
		h6 = x3/6.0 - 0.5*x4 + 0.5*x5 - x6/6.0;
		
		
		res = h6*Gcoeff[k][3];
		res = res * delta;
		res=res + h4*Gcoeff[k][2] + h5*Gcoeff[k+1][2];
		res = res * delta;
		res=res + h2*Gcoeff[k][1] + h3*Gcoeff[k+1][1];
		res = res * delta;
		res=res + h0*Gcoeff[k][0] + h1*Gcoeff[k+1][0];
		
		
		*retVal = res;
}

inline void REBO::subGdif(double cosVal, int mode, double *retVal){
		
	double res;
	double x,x2,x3,x4,x5;
	double hf0,hf1,hf2,hf3,hf4,hf5,hf6;
	int k;
	const double cos0 = 1.0;
	const double cos109 = -0.333411977216052;			//dcos(6.082rad);
	const double cos120 = -0.5;
	const double cos180 = -1.0;
		

		const double Gcoeff[6][4] = {
			{-0.001, 0.10400, 0.0, 0.0},
			{0.05280, 0.17, 0.37, -5.232},
			{0.09733, 0.40, 1.98, 41.6140},
			{8.0, 0.23622, -166.1360, 0.0},
			{0.09733, 0.40, 1.98, -9.95630270},
			{1.0, 0.78, -11.3022275, 0.0}};
		

		double delta;
		
		if (cosVal<=cos120){
			delta = cos120-cos180;
			x = (cosVal - cos180)/delta;
			k = 0;
		}else if (cosVal<=cos109){
			delta = cos109-cos120;
			x = (cosVal - cos120)/delta;
			k = 1;
		}else if (mode == 1) {
			delta = cos0 - cos109;
			x = (cosVal - cos109)/delta;
			k = 2;
		}else if (mode == 2) {
			delta = cos0 - cos109;
			x = (cosVal - cos109)/delta;
			k = 4;
		}else{
			TRACE(_T("error spline G\n"));
		}
		
		
		x2 = x*x;
		x3 = x2*x;
		x4 = x2 * x2;
		x5 = x2 * x3;

		hf0 =  - 60.0*x3 + 120.0*x4 - 60.0*x5;
		hf1 = 60.0*x3 - 120.0*x4 + 60.0*x5;
		hf2 = 1.0 - 40.0*x3 + 75.0*x4 - 36.0*x5;
		hf3 = - 20.0*x3 + 45.0*x4 - 24.0*x5;
		hf4 = x - 12.0*x3 +20.0*x4 - 9.0*x5;
		hf5 = 2.0*x3 - 5.0*x4 + 3.0*x5;
		hf6 = 0.5*x2 - 2.0*x3 + 2.5*x4 - x5;
		
		
		res = hf6*Gcoeff[k][3];
		res = res * delta;
		res=res + hf4*Gcoeff[k][2] + hf5*Gcoeff[k+1][2];
		res = res * delta;
		res=res + hf2*Gcoeff[k][1] + hf3*Gcoeff[k+1][1];
		res=res + (hf0*Gcoeff[k][0] + hf1*Gcoeff[k+1][0])/delta;
		
		*retVal = res;
		
}

//==================================================================;
//;
//		function CalcG;
//;
//;
//;
//									made by Atsushi I.	2004/12/28;
//									Modified by Atsushi I.	2005/06/29;
//==================================================================;

inline double REBO::CalcG(double CosVal){
	
	double tn, t;
	int i;
	double Ret;
		
	//spline coefficient ----------------;
	double Gcoeff[7] = {19.0651, 1.08822, -1.98677, 8.52604, -6.13815, -5.23587, 4.67318};
		
			
	//start routine ---------------------;
		tn = 1.0;
		Ret = 0.0;
		t = CosVal;
		for( i = 0; i <= 6; i++){
			Ret += Gcoeff[i] * tn;
			tn *= t;
		}
	
		return Ret;
}

inline double REBO::CalcGdif(double CosVal){

	//local variant ---------------------;
		double tn, t;
		int i;
		double Ret;
		
	//spline coefficient ----------------;
		const double Gcoeff[7] = {19.0651, 1.08822, -1.98677, 8.52604, -6.13815, -5.23587, 4.67318};
		

	//start routine ---------------------;
		tn = 1.0;
		Ret = 0.0;
		t = CosVal;
		for(i = 1; i<=6;i++){
			
			Ret += (double)i * Gcoeff[i] * tn;
			tn *= t;
		}
	
		return Ret;
}



//======================================================================;
//;
//		function SplineP;
//;
//			this routine calculate bicubic spline P;
//			x: coordination of Hydrogen;
//			y: coordination of Carbon;
//			but this coordination is omitted partner of interaction.;
//									made by Atsushi I.	2004/01/03;
//======================================================================;


inline double REBO::SplineP_2(double x, double y, int PartnerKind){
	
	//local variable -----------------------------;
	
	int ix,iy;
	double xx,yy,xx2,xx3,yy2,yy3;
	double hx0, hx1,hy0, hy1;
	double Ret;
		

	ix = (int)(x);
	iy = (int)(y);
	Ret = 0.0;
		
	if (ix >= SP_PMAXX) {

		if (iy >= SP_PMAXY) {

			Ret = g_spP[SP_PMAXX][SP_PMAXY][PartnerKind];
			
		}else{
			yy = y - iy;
			yy2 = yy * yy;
			yy3 = yy2 * yy;
			hy0 = 1.0 - 3.0 * yy2 + 2.0 * yy3;
			hy1 = 3.0 * yy2 - 2.0 * yy3;
				
			Ret = g_spP[SP_PMAXX][iy][PartnerKind] * hy0;
			Ret = Ret + g_spP[SP_PMAXX][iy + 1][PartnerKind] * hy1;

		}
	}else{
		xx = x - ix;
		xx2 = xx * xx;
		xx3 = xx2 * xx;
		hx0 = 1.0 - 3.0 * xx2 + 2.0 * xx3;	//h(x=0);
		hx1 = 3.0 * xx2 - 2.0 * xx3;			//h(x=1);

		if (iy >= SP_PMAXY) {


			Ret = g_spP[ix][SP_PMAXY][PartnerKind] * hx0;
			Ret = Ret + g_spP[ix + 1][SP_PMAXY][PartnerKind] * hx1;

		}else{
			yy = y - iy;
			yy2 = yy * yy;
			yy3 = yy2 * yy;
			hy0 = 1.0 - 3.0 * yy2 + 2.0 * yy3;
			hy1 = 3.0 * yy2 - 2.0 * yy3;
				
			Ret = g_spP[ix][iy][PartnerKind] * hx0 * hy0;
			Ret += g_spP[ix + 1][iy][PartnerKind] * hx1 * hy0;
			Ret += g_spP[ix][iy + 1][PartnerKind] * hx0 * hy1;
			Ret += g_spP[ix + 1][iy + 1][PartnerKind] * hx1 * hy1;

		}
	}

	return Ret;
}


inline double REBO::SplinePdx_2(double x, double y, int PartnerKind){
		
	//local variable -----------------------------;
		
	int ix,iy;
	double xx,yy,xx2,xx3,yy2,yy3;
	double hx0, hx1,hy0, hy1;
	double Ret;
		

	ix = (int)(x);
	iy = (int)(y);
	Ret = 0.0;
	
	if (ix >= SP_PMAXX) {

	}else{
		xx = x - (double)ix;
		xx2 = xx * xx;
		xx3 = xx2 * xx;
		hx0 = - 6.0 * xx + 6.0 * xx2;	//h(x=0);
		hx1 = 6.0 * xx - 6.0 * xx2;			//h(x=1);
		
		if (iy >= SP_PMAXY) {

			Ret = Ret + g_spP[ix][SP_PMAXY][PartnerKind] * hx0;
			Ret = Ret + g_spP[ix + 1][SP_PMAXY][PartnerKind] * hx1;
			
		}else{
			yy = y - (double)iy;
			yy2 = yy * yy;
			yy3 = yy2 * yy;
			hy0 = 1.0 - 3.0 * yy2 + 2.0 * yy3;
			hy1 = 3.0 * yy2 - 2.0 * yy3;
				
			Ret = g_spP[ix][iy][PartnerKind] * hx0 * hy0;
			Ret = Ret + g_spP[ix + 1][iy][PartnerKind] * hx1 * hy0;
			Ret = Ret + g_spP[ix][iy + 1][PartnerKind] * hx0 * hy1;
			Ret = Ret + g_spP[ix + 1][iy + 1][PartnerKind] * hx1 * hy1;
			
		}
	}
	
	return Ret;
}


inline double REBO::SplinePdy_2(double x, double y, int PartnerKind){
		
	//local variable -----------------------------;
	
	int ix,iy;
	double xx,yy,xx2,xx3,yy2,yy3;
	double hx0, hx1,hy0, hy1;
	double Ret;
	
	ix = (int)(x);
	iy = (int)(y);
	Ret = 0.0;
	
	if (iy >= SP_PMAXY) {

	}else{
		yy = y - (double)iy;
		yy2 = yy * yy;
		yy3 = yy2 * yy;
		hy0 = -6.0 * yy + 6.0 * yy2;
		hy1 = 6.0 * yy - 6.0 * yy2;
			
		if (ix >= SP_PMAXX) {

			Ret = g_spP[SP_PMAXX][iy][PartnerKind] * hy0;
			Ret = Ret + g_spP[SP_PMAXX][iy + 1][PartnerKind] * hy1;
		}else{
			xx = x - (double)ix;
			xx2 = xx * xx;
			xx3 = xx2 * xx;
			hx0 = 1.0 - 3.0 * xx2 + 2.0 * xx3;	//h(x=0);
			hx1 = 3.0 * xx2 - 2.0 * xx3;			//h(x=1);
				
			Ret = g_spP[ix][iy][PartnerKind] * hx0 * hy0;
			Ret = Ret + g_spP[ix + 1][iy][PartnerKind] * hx1 * hy0;
			Ret = Ret + g_spP[ix][iy + 1][PartnerKind] * hx0 * hy1;
			Ret = Ret + g_spP[ix + 1][iy + 1][PartnerKind] * hx1 * hy1;
		}
	}
	
	return Ret;
}


//======================================================================;
//;
//		function SplineF;
//;
//			this routine calculate bicubic spline F;
//;
//;
//;
//									made by Atsushi I.	2004/01/03;
//======================================================================;

inline double REBO::SplineF_2(double x, double y, double z, int PairKind){


	//local variable -----------------------------;
	
	int ix,iy,iz;
	double xx,xx2,xx3,yy,yy2,yy3,zz,zz2,zz3;
	double hx0, hx1, hy0, hy1,hz0, hz1;
	double dhx0, dhx1, dhy0, dhy1,dhz0, dhz1;
	double Ret,temp0,temp1;
	
	ix = (int)(x);
	iy = (int)(y);
	iz = (int)(z);
	Ret = 0.0;
		
	if (ix >= SP_FMAXX) {

		if (iy >= SP_FMAXY) {

			if (iz >= SP_FMAXZ) {

				Ret = g_spF[SP_FMAXX][SP_FMAXY][SP_FMAXZ][PairKind];
					
			}else{
				zz = z - (double)(iz);
				zz2 = zz * zz;
				zz3 = zz2 * zz;
				hz0 = 1.0 - 3.0 * zz2 + 2.0 * zz3;
				hz1 = 3.0 * zz2 - 2.0 * zz3;
				dhz0 = zz - 2.0 * zz2 + zz3;
				dhz1 = - zz2 + zz3;
				
				Ret = g_spF[SP_FMAXX][SP_FMAXY][iz][PairKind]*hz0;
				Ret = Ret + g_spF[SP_FMAXX][SP_FMAXY][iz+1][PairKind]*hz1;
					
					
				if (PairKind == KIND_CC) {
					Ret = Ret + g_spFdz[SP_FMAXX][SP_FMAXY][iz]*dhz0;
					Ret = Ret + g_spFdz[SP_FMAXX][SP_FMAXY][iz+1]*dhz1;
				}
				
			}
		
				
		}else{
			yy = y - (double)(iy);
			yy2 = yy * yy;
			yy3 = yy2 * yy;
			hy0 = 1.0 - 3.0 * yy2 + 2.0 * yy3;
			hy1 = 3.0 * yy2 - 2.0 * yy3;
			dhy0 = yy - 2.0 * yy2 + yy3;
			dhy1 = - yy2 + yy3;
			
				
			if (iz >= SP_FMAXZ) {

				Ret = g_spF[SP_FMAXX][iy][SP_FMAXZ][PairKind]*hy0;
				Ret = Ret + g_spF[SP_FMAXX][iy+1][SP_FMAXZ][PairKind]*hy1;
				
				if (PairKind == KIND_CC) {
					Ret = Ret + g_spFdy[SP_FMAXX][iy][SP_FMAXZ]*dhy0;
					Ret = Ret + g_spFdy[SP_FMAXX][iy+1][SP_FMAXZ]*dhy1;
				}
			}else{
				zz = z - (double)(iz);
				zz2 = zz * zz;
				zz3 = zz2 * zz;
				hz0 = 1.0 - 3.0 * zz2 + 2.0 * zz3;
				hz1 = 3.0 * zz2 - 2.0 * zz3;
				dhz0 = zz - 2.0 * zz2 + zz3;
				dhz1 = - zz2 + zz3;
				;
				temp0 = g_spF[SP_FMAXX][iy][iz][ PairKind]*hy0;
				temp0 = temp0 + g_spF[SP_FMAXX][iy+1][iz][ PairKind]*hy1;
				temp0 = temp0 * hz0;
				;
				temp1 = g_spF[SP_FMAXX][iy][iz+1][ PairKind]*hy0;
				temp1 = temp1 + g_spF[SP_FMAXX][iy+1][iz+1][ PairKind]*hy1;
				temp1 = temp1 * hz1;
				;
				Ret = Ret + temp0 + temp1;
				;
				if (PairKind == KIND_CC) {
					temp0 = g_spFdy[SP_FMAXX][iy][iz]*dhy0;
					temp0 = temp0 + g_spFdy[SP_FMAXX][iy+1][iz]*dhy1;
					temp0 = temp0 * hz0;
					;
					temp1 = g_spFdy[SP_FMAXX][iy][iz+1]*dhy0;
					temp1 = temp1 + g_spFdy[SP_FMAXX][iy+1][iz+1]*dhy1;
					temp1 = temp1 * hz1;
					;
					Ret = Ret + temp0 + temp1;
					;
					temp0 = g_spFdz[SP_FMAXX][iy][iz]*hy0;
					temp0 = temp0 + g_spFdz[SP_FMAXX][iy+1][iz]*hy1;
					temp0 = temp0 * dhz1;
					;
					temp1 = g_spFdz[SP_FMAXX][iy][iz+1]*hy0;
					temp1 = temp1 + g_spFdz[SP_FMAXX][iy+1][iz+1]*hy1;
					temp1 = temp1 * dhz1;
					;
					Ret = Ret + temp0 + temp1;
					;
//					Ret = Ret + g_spFdy[SP_FMAXX][iy+ky][iz+kz]*dhy(ky)*hz(kz);
//					Ret = Ret + g_spFdz[SP_FMAXX][iy+ky][iz+kz]*hy(ky)*dhz(kz);
				}
			
			}
		}
	}else{
			xx = x - (double)(ix);
			xx2 = xx * xx;
			xx3 = xx2 * xx;
			hx0 = 1.0 - 3.0 * xx2 + 2.0 * xx3;	//h(x=0);
			hx1 = 3.0 * xx2 - 2.0 * xx3;			//h(x=1);
			dhx0 = xx - 2.0 * xx2 + xx3;				//hvar(x=0);
			dhx1 = - xx2 + xx3;				//hvar(x=1);
			
			if (iy >= SP_FMAXY) {

				if (iz >= SP_FMAXZ) {

					
					Ret = g_spF[ix][SP_FMAXY][SP_FMAXZ][ PairKind]*hx0;
					Ret = Ret + g_spF[ix+1][SP_FMAXY][SP_FMAXZ][ PairKind]*hx1;
					
					if (PairKind == KIND_CC) {
						Ret = Ret + g_spFdx[ix][SP_FMAXY][SP_FMAXZ]*dhx0;
						Ret = Ret + g_spFdx[ix+1][SP_FMAXY][SP_FMAXZ]*dhx1;
					}
				}else{
					zz = z - (double)(iz);
					zz2 = zz * zz;
					zz3 = zz2 * zz;
					hz0 = 1.0 - 3.0 * zz2 + 2.0 * zz3;
					hz1 = 3.0 * zz2 - 2.0 * zz3;
					dhz0 = zz - 2.0 * zz2 + zz3;
					dhz1 = - zz2 + zz3;
				
					temp0 = g_spF[ix][SP_FMAXY][iz][ PairKind]*hx0;
					temp0 = temp0 + g_spF[ix+1][SP_FMAXY][iz][ PairKind]*hx1;
					temp0 = temp0 * hz0;
					
					temp1 = g_spF[ix][SP_FMAXY][iz+1][ PairKind]*hx0;
					temp1 = temp1 + g_spF[ix+1][SP_FMAXY][iz+1][ PairKind]*hx1;
					temp1 = temp1 * hz1;
					
					Ret = Ret + temp0 + temp1;
					
					if (PairKind == KIND_CC) {
						temp0 = g_spFdx[ix][SP_FMAXY][iz]*dhx0;
						temp0 = temp0 + g_spFdx[ix+1][SP_FMAXY][iz]*dhx1;
						temp0 = temp0 * hz0;
						;
						temp1 = g_spFdx[ix][SP_FMAXY][iz+1]*dhx0;
						temp1 = temp1 + g_spFdx[ix+1][SP_FMAXY][iz+1]*dhx1;
						temp1 = temp1 * hz1;
						;
						Ret = Ret + temp0 + temp1;
						;
						;
						temp0 = g_spFdz[ix][SP_FMAXY][iz]*hx0;
						temp0 = temp0 + g_spFdz[ix+1][SP_FMAXY][iz]*hx1;
						temp0 = temp0 * dhz0;
						;
						temp1 = g_spFdz[ix][SP_FMAXY][iz+1]*hx0;
						temp1 = temp1 + g_spFdz[ix+1][SP_FMAXY][iz+1]*hx1;
						temp1 = temp1 * dhz1;
						;
						Ret = Ret + temp0 + temp1;
						;
	//					Ret = Ret + g_spFdx[ix+kx][iy+ky][iz+kz)*dhx(kx)*hz(kz);
	//					Ret = Ret + g_spFdz[ix+kx][iy+ky][iz+kz)*hx(kx)*dhz(kz);
					}
				}
						
			}else{
				yy = y - (double)(iy);
				yy2 = yy * yy;
				yy3 = yy2 * yy;
				hy0 = 1.0 - 3.0 * yy2 + 2.0 * yy3;
				hy1 = 3.0 * yy2 - 2.0 * yy3;
				dhy0 = yy - 2.0 * yy2 + yy3;
				dhy1 = - yy2 + yy3;
				;
				if (iz >= SP_FMAXZ) {

					temp0 = g_spF[ix][iy][SP_FMAXZ][ PairKind]*hx0;
					temp0 = temp0 + g_spF[ix+1][iy][SP_FMAXZ][ PairKind]*hx1;
					temp0 = temp0 * hy0;
					;
					temp1 = g_spF[ix][iy+1][SP_FMAXZ][ PairKind]*hx0;
					temp1 = temp1 + g_spF[ix+1][iy+1][SP_FMAXZ][ PairKind]*hx1;
					temp1 = temp1 * hy1;
					;
					Ret = Ret + temp0 + temp1;
					;
//					Ret = Ret + g_spF[ix+kx][iy+ky][SP_FMAXZ][ PairKind]*hx(kx)*hy(ky);
					;
					if (PairKind == KIND_CC) {
						;
						temp0 = g_spFdx[ix][iy][SP_FMAXZ]*dhx0;
						temp0 = temp0 + g_spFdx[ix+1][iy][SP_FMAXZ]*dhx1;
						temp0 = temp0 * hy0;
						;
						temp1 = g_spFdx[ix][iy+1][SP_FMAXZ]*dhx0;
						temp1 = temp1 + g_spFdx[ix+1][iy+1][SP_FMAXZ]*dhx1;
						temp1 = temp1 * hy1;
						;
						Ret = Ret + temp0 + temp1;
						;
						;
						temp0 = g_spFdy[ix][iy][SP_FMAXZ]*hx0;
						temp0 = temp0 + g_spFdy[ix+1][iy][SP_FMAXZ]*hx1;
						temp0 = temp0 * dhy0;
						;
						temp1 = g_spFdy[ix][iy+1][SP_FMAXZ]*hx0;
						temp1 = temp1 + g_spFdy[ix+1][iy+1][SP_FMAXZ]*hx1;
						temp1 = temp1 * dhy1;
						;
						Ret = Ret + temp0 + temp1;
						;
						;
						;
	//					Ret = Ret + g_spFdx[ix+kx][iy+ky][SP_FMAXZ]*dhx(kx)*hy(ky);
	//					Ret = Ret + g_spFdy[ix+kx][iy+ky][SP_FMAXZ]*hx(kx)*dhy(ky);
					}
					
				}else{
					zz = z - (double)(iz);
					zz2 = zz * zz;
					zz3 = zz2 * zz;
					hz0 = 1.0 - 3.0 * zz2 + 2.0 * zz3;
					hz1 = 3.0 * zz2 - 2.0 * zz3;
					dhz0 = zz - 2.0 * zz2 + zz3;
					dhz1 = - zz2 + zz3;
					
					
					temp0 = g_spF[ix][iy][iz][ PairKind]*hx0;
					temp0 = temp0 + g_spF[ix+1][iy][iz][ PairKind]*hx1;
					temp0 = temp0 * hy0;
					;
					temp1 = g_spF[ix][iy+1][iz][ PairKind]*hx0;
					temp1 = temp1 + g_spF[ix+1][iy+1][iz][ PairKind]*hx1;
					temp1 = temp1 * hy1;
					;
					Ret = Ret + (temp0 + temp1) * hz0;
					;
					temp0 = g_spF[ix][iy][iz+1][ PairKind]*hx0;
					temp0 = temp0 + g_spF[ix+1][iy][iz+1][ PairKind]*hx1;
					temp0 = temp0 * hy0;
					;
					temp1 = g_spF[ix][iy+1][iz+1][ PairKind]*hx0;
					temp1 = temp1 + g_spF[ix+1][iy+1][iz+1][ PairKind]*hx1;
					temp1 = temp1 * hy1;
					;
					Ret = Ret + (temp0 + temp1) * hz1;
					
					
					if (PairKind == KIND_CC) {
						
						temp0 = g_spFdx[ix][iy][iz]*dhx0;
						temp0 = temp0 + g_spFdx[ix+1][iy][iz]*dhx1;
						temp0 = temp0 * hy0;
						;
						temp1 = g_spFdx[ix][iy+1][iz]*dhx0;
						temp1 = temp1 + g_spFdx[ix+1][iy+1][iz]*dhx1;
						temp1 = temp1 * hy1;
						;
						Ret = Ret + (temp0 + temp1) * hz0;
						;
						temp0 = g_spFdx[ix][iy][iz+1]*dhx0;
						temp0 = temp0 + g_spFdx[ix+1][iy][iz+1]*dhx1;
						temp0 = temp0 * hy0;
						;
						temp1 = g_spFdx[ix][iy+1][iz+1]*dhx0;
						temp1 = temp1 + g_spFdx[ix+1][iy+1][iz+1]*dhx1;
						temp1 = temp1 * hy1;
						;
						Ret = Ret + (temp0 + temp1) * hz1;
				

						temp0 = g_spFdy[ix][iy][iz]*hx0;
						temp0 = temp0 + g_spFdy[ix+1][iy][iz]*hx1;
						temp0 = temp0 * dhy0;
						;
						temp1 = g_spFdy[ix][iy+1][iz]*hx0;
						temp1 = temp1 + g_spFdy[ix+1][iy+1][iz]*hx1;
						temp1 = temp1 * dhy1;
						;
						Ret = Ret + (temp0 + temp1) * hz0;
						;
						temp0 = g_spFdy[ix][iy][iz+1]*hx0;
						temp0 = temp0 + g_spFdy[ix+1][iy][iz+1]*hx1;
						temp0 = temp0 * dhy0;
						;
						temp1 = g_spFdy[ix][iy+1][iz+1]*hx0;
						temp1 = temp1 + g_spFdy[ix+1][iy+1][iz+1]*hx1;
						temp1 = temp1 * dhy1;
						;
						Ret = Ret + (temp0 + temp1) * hz1;
							

						temp0 = g_spFdz[ix][iy][iz]*hx0;
						temp0 = temp0 + g_spFdz[ix+1][iy][iz]*hx1;
						temp0 = temp0 * hy0;
						;
						temp1 = g_spFdz[ix][iy+1][iz]*hx0;
						temp1 = temp1 + g_spFdz[ix+1][iy+1][iz]*hx1;
						temp1 = temp1 * hy1;
						;
						Ret = Ret + (temp0 + temp1) * dhz0;
						;
						temp0 = g_spFdz[ix][iy][iz+1]*hx0;
						temp0 = temp0 + g_spFdz[ix+1][iy][iz+1]*hx1;
						temp0 = temp0 * hy0;
						;
						temp1 = g_spFdz[ix][iy+1][iz+1]*hx0;
						temp1 = temp1 + g_spFdz[ix+1][iy+1][iz+1]*hx1;
						temp1 = temp1 * hy1;
						;
						Ret = Ret + (temp0 + temp1) * dhz1;
					
					}
				}
			}
		}
		
	return Ret;
}

inline double REBO::SplineFdx_2(double x, double y, double z, int PairKind){


	int ix,iy,iz;
	double xx,xx2,xx3,yy,yy2,yy3,zz,zz2,zz3;
	double hx0, hx1,hy0, hy1,hz0, hz1;
	double dhx0, dhx1,dhy0, dhy1,dhz0, dhz1;
	double Ret,temp0,temp1;
	
	ix = (int)(x);
	iy = (int)(y);
	iz = (int)(z);
	Ret = 0.0;
	
	if (ix >= SP_FMAXX) {
	
	}else{
		xx = x - (double)(ix);
		xx2 = xx * xx;
		xx3 = xx2 * xx;
		hx0 = - 6.0 * xx + 6.0 * xx2;	//h(x=0);
		hx1 = 6.0 * xx - 6.0 * xx2;			//h(x=1);
		dhx0 = 1.0 - 4.0 * xx + 3.0 * xx2;				//hvar(x=0);
		dhx1 = - 2.0 * xx + 3.0 * xx2;				//hvar(x=1);
		
		if (iy >= SP_FMAXY) {

			if (iz >= SP_FMAXZ) {

				Ret = g_spF[ix][SP_FMAXY][SP_FMAXZ][ PairKind]*hx0;
				Ret = Ret + g_spF[ix+1][SP_FMAXY][SP_FMAXZ][ PairKind]*hx1;
				
				
				if (PairKind == KIND_CC) {
					Ret = Ret + g_spFdx[ix][SP_FMAXY][SP_FMAXZ]*dhx0;
					Ret = Ret + g_spFdx[ix+1][SP_FMAXY][SP_FMAXZ]*dhx1;
				}
			}else{
				zz = z - (double)(iz);
				zz2 = zz * zz;
				zz3 = zz2 * zz;
				hz0 = 1.0 - 3.0 * zz2 + 2.0 * zz3;
				hz1 = 3.0 * zz2 - 2.0 * zz3;
				dhz0 = zz - 2.0 * zz2 + zz3;
				dhz1 = - zz2 + zz3;
					
				temp0 = g_spF[ix][SP_FMAXY][iz][ PairKind]*hx0;
				temp0 = temp0 + g_spF[ix+1][SP_FMAXY][iz][ PairKind]*hx1;
				temp0 = temp0 * hz0;
				
				temp1 = g_spF[ix][SP_FMAXY][iz+1][ PairKind]*hx0;
				temp1 = temp1 + g_spF[ix+1][SP_FMAXY][iz+1][ PairKind]*hx1;
				temp1 = temp1 * hz1;
				
				Ret = Ret + temp0 + temp1;
				
				if (PairKind == KIND_CC) {
					temp0 = g_spFdx[ix][SP_FMAXY][iz]*dhx0;
					temp0 = temp0 + g_spFdx[ix+1][SP_FMAXY][iz]*dhx1;
					temp0 = temp0 * hz0;
				
					temp1 = g_spFdx[ix][SP_FMAXY][iz+1]*dhx0;
					temp1 = temp1 + g_spFdx[ix+1][SP_FMAXY][iz+1]*dhx1;
					temp1 = temp1 * hz1;
				
					Ret = Ret + temp0 + temp1;
				
					temp0 = g_spFdz[ix][SP_FMAXY][iz]*hx0;
					temp0 = temp0 + g_spFdz[ix+1][SP_FMAXY][iz]*hx1;
					temp0 = temp0 * dhz0;
					
					temp1 = g_spFdz[ix][SP_FMAXY][iz+1]*hx0;
					temp1 = temp1 + g_spFdz[ix+1][SP_FMAXY][iz+1]*hx1;
					temp1 = temp1 * dhz1;
					
					Ret = Ret + temp0 + temp1;
				}
			}
		}else{
			yy = y - (double)(iy);
			yy2 = yy * yy;
			yy3 = yy2 * yy;
			hy0 = 1.0 - 3.0 * yy2 + 2.0 * yy3;
			hy1 = 3.0 * yy2 - 2.0 * yy3;
			dhy0 = yy - 2.0 * yy2 + yy3;
			dhy1 = - yy2 + yy3;
				
			if (iz >= SP_FMAXZ) {

				temp0 = g_spF[ix][iy][SP_FMAXZ][ PairKind]*hx0;
				temp0 = temp0 + g_spF[ix+1][iy][SP_FMAXZ][ PairKind]*hx1;
				temp0 = temp0 * hy0;
				
				temp1 = g_spF[ix][iy+1][SP_FMAXZ][ PairKind]*hx0;
				temp1 = temp1 + g_spF[ix+1][iy+1][SP_FMAXZ][ PairKind]*hx1;
				temp1 = temp1 * hy1;
					
				Ret = Ret + temp0 + temp1;
				
				if (PairKind == KIND_CC) {
					temp0 = g_spFdx[ix][iy][SP_FMAXZ]*dhx0;
					temp0 = temp0 + g_spFdx[ix+1][iy][SP_FMAXZ]*dhx1;
					temp0 = temp0 * hy0;
				
					temp1 = g_spFdx[ix][iy+1][SP_FMAXZ]*dhx0;
					temp1 = temp1 + g_spFdx[ix+1][iy+1][SP_FMAXZ]*dhx1;
					temp1 = temp1 * hy1;
				
					Ret = Ret + temp0 + temp1;
				
					temp0 = g_spFdy[ix][iy][SP_FMAXZ]*hx0;
					temp0 = temp0 + g_spFdy[ix+1][iy][SP_FMAXZ]*hx1;
					temp0 = temp0 * dhy0;
					
					temp1 = g_spFdy[ix][iy+1][SP_FMAXZ]*hx0;
					temp1 = temp1 + g_spFdy[ix+1][iy+1][SP_FMAXZ]*hx1;
					temp1 = temp1 * dhy1;
					
					Ret = Ret + temp0 + temp1;
				}
			}else{
				zz = z - (double)(iz);
				zz2 = zz * zz;
				zz3 = zz2 * zz;
				hz0 = 1.0 - 3.0 * zz2 + 2.0 * zz3;
				hz1 = 3.0 * zz2 - 2.0 * zz3;
				dhz0 = zz - 2.0 * zz2 + zz3;
				dhz1 = - zz2 + zz3;
					
				temp0 = g_spF[ix][iy][iz][ PairKind]*hx0;
				temp0 = temp0 + g_spF[ix+1][iy][iz][ PairKind]*hx1;
				temp0 = temp0 * hy0;
					
				temp1 = g_spF[ix][iy+1][iz][ PairKind]*hx0;
				temp1 = temp1 + g_spF[ix+1][iy+1][iz][ PairKind]*hx1;
				temp1 = temp1 * hy1;
					
				Ret = Ret + (temp0 + temp1) * hz0;
				
				temp0 = g_spF[ix][iy][iz+1][ PairKind]*hx0;
				temp0 = temp0 + g_spF[ix+1][iy][iz+1][ PairKind]*hx1;
				temp0 = temp0 * hy0;
				
				temp1 = g_spF[ix][iy+1][iz+1][ PairKind]*hx0;
				temp1 = temp1 + g_spF[ix+1][iy+1][iz+1][ PairKind]*hx1;
				temp1 = temp1 * hy1;
				
				Ret = Ret + (temp0 + temp1) * hz1;
				
				if (PairKind == KIND_CC) {
				
					temp0 = g_spFdx[ix][iy][iz]*dhx0;
					temp0 = temp0 + g_spFdx[ix+1][iy][iz]*dhx1;
					temp0 = temp0 * hy0;
				
					temp1 = g_spFdx[ix][iy+1][iz]*dhx0;
					temp1 = temp1 + g_spFdx[ix+1][iy+1][iz]*dhx1;
					temp1 = temp1 * hy1;
				
					Ret = Ret + (temp0 + temp1) * hz0;
				
					temp0 = g_spFdx[ix][iy][iz+1]*dhx0;
					temp0 = temp0 + g_spFdx[ix+1][iy][iz+1]*dhx1;
					temp0 = temp0 * hy0;
				
					temp1 = g_spFdx[ix][iy+1][iz+1]*dhx0;
					temp1 = temp1 + g_spFdx[ix+1][iy+1][iz+1]*dhx1;
					temp1 = temp1 * hy1;
				
					Ret = Ret + (temp0 + temp1) * hz1;
				
					temp0 = g_spFdy[ix][iy][iz]*hx0;
					temp0 = temp0 + g_spFdy[ix+1][iy][iz]*hx1;
					temp0 = temp0 * dhy0;
				
					temp1 = g_spFdy[ix][iy+1][iz]*hx0;
					temp1 = temp1 + g_spFdy[ix+1][iy+1][iz]*hx1;
					temp1 = temp1 * dhy1;
				
					Ret = Ret + (temp0 + temp1) * hz0;
				
					temp0 = g_spFdy[ix][iy][iz+1]*hx0;
					temp0 = temp0 + g_spFdy[ix+1][iy][iz+1]*hx1;
					temp0 = temp0 * dhy0;
				
					temp1 = g_spFdy[ix][iy+1][iz+1]*hx0;
					temp1 = temp1 + g_spFdy[ix+1][iy+1][iz+1]*hx1;
					temp1 = temp1 * dhy1;
				
					Ret = Ret + (temp0 + temp1) * hz1;
				
					temp0 = g_spFdz[ix][iy][iz]*hx0;
					temp0 = temp0 + g_spFdz[ix+1][iy][iz]*hx1;
					temp0 = temp0 * hy0;
				
					temp1 = g_spFdz[ix][iy+1][iz]*hx0;
					temp1 = temp1 + g_spFdz[ix+1][iy+1][iz]*hx1;
					temp1 = temp1 * hy1;
				
					Ret = Ret + (temp0 + temp1) * dhz0;
				
					temp0 = g_spFdz[ix][iy][iz+1]*hx0;
					temp0 = temp0 + g_spFdz[ix+1][iy][iz+1]*hx1;
					temp0 = temp0 * hy0;
				
					temp1 = g_spFdz[ix][iy+1][iz+1]*hx0;
					temp1 = temp1 + g_spFdz[ix+1][iy+1][iz+1]*hx1;
					temp1 = temp1 * hy1;
				
					Ret = Ret + (temp0 + temp1) * dhz1;
				}
			}
		}
	}
	return Ret;
}


inline double REBO::SplineFdz_2(double x, double y, double z, int PairKind){

	int ix,iy,iz;
	double xx,xx2,xx3,yy,yy2,yy3,zz,zz2,zz3;
	double hx0, hx1,hy0, hy1,hz0, hz1;
	double dhx0, dhx1,dhy0, dhy1,dhz0, dhz1;
	double Ret,temp0,temp1;
	
	ix = (int)(x);
	iy = (int)(y);
	iz = (int)(z);
	Ret = 0.0;
	
	if (iz >= SP_FMAXZ) {

	}else{
		zz = z - (double)(iz);
		zz2 = zz * zz;
		zz3 = zz2 * zz;
		hz0 = - 6.0 * zz + 6.0 * zz2;
		hz1 = 6.0 * zz - 6.0 * zz2;
		dhz0 = 1.0 - 4.0 * zz + 3.0 * zz2;
		dhz1 = - 2.0 * zz + 3.0 * zz2;
		
		if (ix >= SP_FMAXX) {
	
			if (iy >= SP_FMAXY) {
	
				Ret = g_spF[SP_FMAXX][SP_FMAXY][iz][ PairKind]*hz0;
				Ret = Ret + g_spF[SP_FMAXX][SP_FMAXY][iz+1][ PairKind]*hz1;
				
				if (PairKind == KIND_CC) {
					Ret = Ret + g_spFdz[SP_FMAXX][SP_FMAXY][iz]*dhz0;
					Ret = Ret + g_spFdz[SP_FMAXX][SP_FMAXY][iz+1]*dhz1;
				}
			}else{
				yy = y - (double)(iy);
				yy2 = yy * yy;
				yy3 = yy2 * yy;
				hy0 = 1.0 - 3.0 * yy2 + 2.0 * yy3;
				hy1 = 3.0 * yy2 - 2.0 * yy3;
				dhy0 = yy - 2.0 * yy2 + yy3;
				dhy1 = - yy2 + yy3;
				
				
				temp0 = g_spF[SP_FMAXX][iy][iz][ PairKind]*hy0;
				temp0 = temp0 + g_spF[SP_FMAXX][iy+1][iz][ PairKind]*hy1;
				temp0 = temp0 * hz0;
					
				temp1 = g_spF[SP_FMAXX][iy][iz+1][ PairKind]*hy0;
				temp1 = temp1 + g_spF[SP_FMAXX][iy+1][iz+1][ PairKind]*hy1;
				temp1 = temp1 * hz1;
					
				Ret = Ret + temp0 + temp1;
				
				if (PairKind == KIND_CC) {
					temp0 = g_spFdy[SP_FMAXX][iy][iz]*dhy0;
					temp0 = temp0 + g_spFdy[SP_FMAXX][iy+1][iz]*dhy1;
					temp0 = temp0 * hz0;
			
					temp1 = g_spFdy[SP_FMAXX][iy][iz+1]*dhy0;
					temp1 = temp1 + g_spFdy[SP_FMAXX][iy+1][iz+1]*dhy1;
					temp1 = temp1 * hz1;
					
					Ret = Ret + temp0 + temp1;
					
					temp0 = g_spFdz[SP_FMAXX][iy][iz]*hy0;
					temp0 = temp0 + g_spFdz[SP_FMAXX][iy+1][iz]*hy1;
					temp0 = temp0 * dhz1;
					
					temp1 = g_spFdz[SP_FMAXX][iy][iz+1]*hy0;
					temp1 = temp1 + g_spFdz[SP_FMAXX][iy+1][iz+1]*hy1;
					temp1 = temp1 * dhz1;
					
					Ret = Ret + temp0 + temp1;
				}
			}
			
		}else{
			xx = x - (double)(ix);
			xx2 = xx * xx;
			xx3 = xx2 * xx;
			hx0 = 1.0 - 3.0 * xx2 + 2.0 * xx3;	//h(x=0);
			hx1 = 3.0 * xx2 - 2.0 * xx3;			//h(x=1);
			dhx0 = xx - 2.0 * xx2 + xx3;			//hvar(x=0);
			dhx1 = - xx2 + xx3;					//hvar(x=1);
				
				
			if (iy >= SP_FMAXY) {
	
				temp0 = g_spF[ix][SP_FMAXY][iz][ PairKind]*hx0;
				temp0 = temp0 + g_spF[ix+1][SP_FMAXY][iz][ PairKind]*hx1;
				temp0 = temp0 * hz0;
					
				temp1 = g_spF[ix][SP_FMAXY][iz+1][ PairKind]*hx0;
				temp1 = temp1 + g_spF[ix+1][SP_FMAXY][iz+1][ PairKind]*hx1;
				temp1 = temp1 * hz1;
					
				Ret = Ret + temp0 + temp1;
				
				if (PairKind == KIND_CC) {
					temp0 = g_spFdx[ix][SP_FMAXY][iz]*dhx0;
					temp0 = temp0 + g_spFdx[ix+1][SP_FMAXY][iz]*dhx1;
					temp0 = temp0 * hz0;
					
					temp1 = g_spFdx[ix][SP_FMAXY][iz+1]*dhx0;
					temp1 = temp1 + g_spFdx[ix+1][SP_FMAXY][iz+1]*dhx1;
					temp1 = temp1 * hz1;
					
					Ret = Ret + temp0 + temp1;
					
					temp0 = g_spFdz[ix][SP_FMAXY][iz]*hx0;
					temp0 = temp0 + g_spFdz[ix+1][SP_FMAXY][iz]*hx1;
					temp0 = temp0 * dhz0;
					
					temp1 = g_spFdz[ix][SP_FMAXY][iz+1]*hx0;
					temp1 = temp1 + g_spFdz[ix+1][SP_FMAXY][iz+1]*hx1;
					temp1 = temp1 * dhz1;
					
					Ret = Ret + temp0 + temp1;
				}
			}else{
				yy = y - (double)(iy);
				yy2 = yy * yy;
				yy3 = yy2 * yy;
				hy0 = 1.0 - 3.0 * yy2 + 2.0 * yy3;
				hy1 = 3.0 * yy2 - 2.0 * yy3;
				dhy0 = yy - 2.0 * yy2 + yy3;
				dhy1 = - yy2 + yy3;
				
				temp0 = g_spF[ix][iy][iz][ PairKind]*hx0;
				temp0 = temp0 + g_spF[ix+1][iy][iz][ PairKind]*hx1;
				temp0 = temp0 * hy0;
				
				temp1 = g_spF[ix][iy+1][iz][ PairKind]*hx0;
				temp1 = temp1 + g_spF[ix+1][iy+1][iz][ PairKind]*hx1;
				temp1 = temp1 * hy1;
					
				Ret = Ret + (temp0 + temp1) * hz0;
				
				temp0 = g_spF[ix][iy][iz+1][ PairKind]*hx0;
				temp0 = temp0 + g_spF[ix+1][iy][iz+1][ PairKind]*hx1;
				temp0 = temp0 * hy0;
				
				temp1 = g_spF[ix][iy+1][iz+1][ PairKind]*hx0;
				temp1 = temp1 + g_spF[ix+1][iy+1][iz+1][ PairKind]*hx1;
				temp1 = temp1 * hy1;
				
				Ret = Ret + (temp0 + temp1) * hz1;
				
				if (PairKind == KIND_CC) {
				
					temp0 = g_spFdx[ix][iy][iz]*dhx0;
					temp0 = temp0 + g_spFdx[ix+1][iy][iz]*dhx1;
					temp0 = temp0 * hy0;
				
					temp1 = g_spFdx[ix][iy+1][iz]*dhx0;
					temp1 = temp1 + g_spFdx[ix+1][iy+1][iz]*dhx1;
					temp1 = temp1 * hy1;
				
					Ret = Ret + (temp0 + temp1) * hz0;
				
					temp0 = g_spFdx[ix][iy][iz+1]*dhx0;
					temp0 = temp0 + g_spFdx[ix+1][iy][iz+1]*dhx1;
					temp0 = temp0 * hy0;
				
					temp1 = g_spFdx[ix][iy+1][iz+1]*dhx0;
					temp1 = temp1 + g_spFdx[ix+1][iy+1][iz+1]*dhx1;
					temp1 = temp1 * hy1;
				
					Ret = Ret + (temp0 + temp1) * hz1;
				
					
					temp0 = g_spFdy[ix][iy][iz]*hx0;
					temp0 = temp0 + g_spFdy[ix+1][iy][iz]*hx1;
					temp0 = temp0 * dhy0;
					
					temp1 = g_spFdy[ix][iy+1][iz]*hx0;
					temp1 = temp1 + g_spFdy[ix+1][iy+1][iz]*hx1;
					temp1 = temp1 * dhy1;
					
					Ret = Ret + (temp0 + temp1) * hz0;
					
					temp0 = g_spFdy[ix][iy][iz+1]*hx0;
					temp0 = temp0 + g_spFdy[ix+1][iy][iz+1]*hx1;
					temp0 = temp0 * dhy0;
					
					temp1 = g_spFdy[ix][iy+1][iz+1]*hx0;
					temp1 = temp1 + g_spFdy[ix+1][iy+1][iz+1]*hx1;
					temp1 = temp1 * dhy1;
					
					Ret = Ret + (temp0 + temp1) * hz1;
					
					temp0 = g_spFdz[ix][iy][iz]*hx0;
					temp0 = temp0 + g_spFdz[ix+1][iy][iz]*hx1;
					temp0 = temp0 * hy0;
					
					temp1 = g_spFdz[ix][iy+1][iz]*hx0;
					temp1 = temp1 + g_spFdz[ix+1][iy+1][iz]*hx1;
					temp1 = temp1 * hy1;
					
					Ret = Ret + (temp0 + temp1) * dhz0;
					
					temp0 = g_spFdz[ix][iy][iz+1]*hx0;
					temp0 = temp0 + g_spFdz[ix+1][iy][iz+1]*hx1;
					temp0 = temp0 * hy0;
					
					temp1 = g_spFdz[ix][iy+1][iz+1]*hx0;
					temp1 = temp1 + g_spFdz[ix+1][iy+1][iz+1]*hx1;
					temp1 = temp1 * hy1;
					
					Ret = Ret + (temp0 + temp1) * dhz1;
					
				}
			}
		}
	}
	return Ret;
}

//======================================================================;
//;
//		function SplineT;
//;
//			this routine calculate bicubic spline T;
//;
//;
//;
//									made by Atsushi I.	2004/01/03;
//======================================================================;

inline double REBO::SplineT_2(double x, double y, double z){

	//local variable -----------------------------;
		
	int ix,iy,iz;
	double xx,xx2,xx3,yy,yy2,yy3,zz,zz2,zz3;
	double hx0, hx1,hy0, hy1,hz0, hz1;
	double Ret,temp0,temp1;
	
	ix = (int)(x);
	iy = (int)(y);
	iz = (int)(z);
	Ret = 0.0;
	
	if (ix >= SP_TMAXX) {

		if (iy >= SP_TMAXY) {

			if (iz >= SP_TMAXZ) {

				Ret = g_spT[SP_TMAXX][SP_TMAXY][SP_TMAXZ];
					
			}else{
				zz = z - (double)(iz);
				zz2 = zz * zz;
				zz3 = zz2 * zz;
				hz0 = 1.0 - 3.0 * zz2 + 2.0 * zz3;
				hz1 = 3.0 * zz2 - 2.0 * zz3;
				
				Ret = g_spT[SP_TMAXX][SP_TMAXY][iz]*hz0;
				Ret = Ret + g_spT[SP_TMAXX][SP_TMAXY][iz+1]*hz1;
			}
			
		}else{
			yy = y - (double)(iy);
			yy2 = yy * yy;
			yy3 = yy2 * yy;
			hy0 = 1.0 - 3.0 * yy2 + 2.0 * yy3;
			hy1 = 3.0 * yy2 - 2.0 * yy3;
				
			if (iz >= SP_TMAXZ) {
				Ret = g_spT[SP_TMAXX][iy][SP_TMAXZ]*hy0;
				Ret = Ret + g_spT[SP_TMAXX][iy+1][SP_TMAXZ]*hy1;

			}else{
				zz = z - (double)(iz);
				zz2 = zz * zz;
				zz3 = zz2 * zz;
				hz0 = 1.0 - 3.0 * zz2 + 2.0 * zz3;
				hz1 = 3.0 * zz2 - 2.0 * zz3;
				
				temp0 = g_spT[SP_TMAXX][iy][iz]*hy0;
				temp0 = temp0 + g_spT[SP_TMAXX][iy+1][iz]*hy1;
				temp0 = temp0 * hz0;
					
				temp1 = g_spT[SP_TMAXX][iy][iz+1]*hy0;
				temp1 = temp1 + g_spT[SP_TMAXX][iy+1][iz+1]*hy1;
				temp1 = temp1 * hz1;
					
				Ret = Ret + temp0 + temp1;

			}
		}
	}else{
		xx = x - (double)(ix);
		xx2 = xx * xx;
		xx3 = xx2 * xx;
		hx0 = 1.0 - 3.0 * xx2 + 2.0 * xx3;	//h(x=0);
		hx1 = 3.0 * xx2 - 2.0 * xx3;			//h(x=1);
			
		if (iy >= SP_TMAXY) {

			if (iz >= SP_TMAXZ) {

				Ret = g_spT[ix][SP_TMAXY][SP_TMAXZ]*hx0;
				Ret = Ret + g_spT[ix+1][SP_TMAXY][SP_TMAXZ]*hx1;
				
			}else{
				zz = z - (double)(iz);
				zz2 = zz * zz;
				zz3 = zz2 * zz;
				hz0 = 1.0 - 3.0 * zz2 + 2.0 * zz3;
				hz1 = 3.0 * zz2 - 2.0 * zz3;
				
				temp0 = g_spT[ix][SP_TMAXY][iz]*hx0;
				temp0 = temp0 + g_spT[ix+1][SP_TMAXY][iz]*hx1;
				temp0 = temp0 * hz0;
					
				temp1 = g_spT[ix][SP_TMAXY][iz+1]*hx0;
				temp1 = temp1 + g_spT[ix+1][SP_TMAXY][iz+1]*hx1;
				temp1 = temp1 * hz1;
					
				Ret = Ret + temp0 + temp1;
			}
		}else{
			yy = y - (double)(iy);
			yy2 = yy * yy;
			yy3 = yy2 * yy;
			hy0 = 1.0 - 3.0 * yy2 + 2.0 * yy3;
			hy1 = 3.0 * yy2 - 2.0 * yy3;
				
			if (iz >= SP_TMAXZ) {

				temp0 = g_spT[ix][iy][SP_TMAXZ]*hx0;
				temp0 = temp0 + g_spT[ix+1][iy][SP_TMAXZ]*hx1;
				temp0 = temp0 * hy0;
					
				temp1 = g_spT[ix][iy+1][SP_TMAXZ]*hx0;
				temp1 = temp1 + g_spT[ix+1][iy+1][SP_TMAXZ]*hx1;
				temp1 = temp1 * hy1;
				
				Ret = Ret + temp0 + temp1;
				
			}else{
				zz = z - (double)(iz);
				zz2 = zz * zz;
				zz3 = zz2 * zz;
				hz0 = 1.0 - 3.0 * zz2 + 2.0 * zz3;
				hz1 = 3.0 * zz2 - 2.0 * zz3;
					
					
				temp0 = g_spT[ix][iy][iz]*hx0;
				temp0 = temp0 + g_spT[ix+1][iy][iz]*hx1;
				temp0 = temp0 * hy0;
					
				temp1 = g_spT[ix][iy+1][iz]*hx0;
				temp1 = temp1 + g_spT[ix+1][iy+1][iz]*hx1;
				temp1 = temp1 * hy1;
					
				Ret = Ret + (temp0 + temp1) * hz0;
				
				temp0 = g_spT[ix][iy][iz+1]*hx0;
				temp0 = temp0 + g_spT[ix+1][iy][iz+1]*hx1;
				temp0 = temp0 * hy0;
					
				temp1 = g_spT[ix][iy+1][iz+1]*hx0;
				temp1 = temp1 + g_spT[ix+1][iy+1][iz+1]*hx1;
				temp1 = temp1 * hy1;
					
				Ret = Ret + (temp0 + temp1) * hz1;
					

			}
		}
	}
		
	return Ret;
}

inline double REBO::SplineTdx_2(double x, double y, double z){


	int ix,iy,iz;
	double xx,xx2,xx3,yy,yy2,yy3,zz,zz2,zz3;
	double hx0, hx1,hy0, hy1,hz0, hz1;
	double Ret,temp0,temp1;
	
	ix = (int)(x);
	iy = (int)(y);
	iz = (int)(z);
	Ret = 0.0;
	
	if (ix >= SP_TMAXX) {

	}else{
		xx = x - (double)(ix);
		xx2 = xx * xx;
		xx3 = xx2 * xx;
		hx0 = - 6.0 * xx + 6.0 * xx2;	//h(x=0);
		hx1 = 6.0 * xx - 6.0 * xx2;			//h(x=1);


		if (iy >= SP_TMAXY) {
			if (iz >= SP_TMAXZ) {

				Ret = g_spT[ix][SP_TMAXY][SP_TMAXZ]*hx0;
				Ret = Ret + g_spT[ix+1][SP_TMAXY][SP_TMAXZ]*hx1;
			}else{
				zz = z - (double)(iz);
				zz2 = zz * zz;
				zz3 = zz2 * zz;
				hz0 = 1.0 - 3.0 * zz2 + 2.0 * zz3;
				hz1 = 3.0 * zz2 - 2.0 * zz3;
					
				temp0 = g_spT[ix][SP_TMAXY][iz]*hx0;
				temp0 = temp0 + g_spT[ix+1][SP_TMAXY][iz]*hx1;
				temp0 = temp0 * hz0;
					
				temp1 = g_spT[ix][SP_TMAXY][iz+1]*hx0;
				temp1 = temp1 + g_spT[ix+1][SP_TMAXY][iz+1]*hx1;
				temp1 = temp1 * hz1;
					
				Ret = Ret + temp0 + temp1;
			}

		}else{
			yy = y - (double)(iy);
			yy2 = yy * yy;
			yy3 = yy2 * yy;
			hy0 = 1.0 - 3.0 * yy2 + 2.0 * yy3;
			hy1 = 3.0 * yy2 - 2.0 * yy3;

			if (iz >= SP_TMAXZ) {

				temp0 = g_spT[ix][iy][SP_TMAXZ]*hx0;
				temp0 = temp0 + g_spT[ix+1][iy][SP_TMAXZ]*hx1;
				temp0 = temp0 * hy0;
				
				temp1 = g_spT[ix][iy+1][SP_TMAXZ]*hx0;
				temp1 = temp1 + g_spT[ix+1][iy+1][SP_TMAXZ]*hx1;
				temp1 = temp1 * hy1;
				
				Ret = Ret + temp0 + temp1;
			}else{
				zz = z - (double)(iz);
				zz2 = zz * zz;
				zz3 = zz2 * zz;
				hz0 = 1.0 - 3.0 * zz2 + 2.0 * zz3;
				hz1 = 3.0 * zz2 - 2.0 * zz3;
					
				temp0 = g_spT[ix][iy][iz]*hx0;
				temp0 = temp0 + g_spT[ix+1][iy][iz]*hx1;
				temp0 = temp0 * hy0;
					
				temp1 = g_spT[ix][iy+1][iz]*hx0;
				temp1 = temp1 + g_spT[ix+1][iy+1][iz]*hx1;
				temp1 = temp1 * hy1;
				
				Ret = Ret + (temp0 + temp1) * hz0;
				
				temp0 = g_spT[ix][iy][iz+1]*hx0;
				temp0 = temp0 + g_spT[ix+1][iy][iz+1]*hx1;
				temp0 = temp0 * hy0;
					
				temp1 = g_spT[ix][iy+1][iz+1]*hx0;
				temp1 = temp1 + g_spT[ix+1][iy+1][iz+1]*hx1;
				temp1 = temp1 * hy1;
				
				Ret = Ret + (temp0 + temp1) * hz1;
			}
		}
	}
	return Ret;
}


inline double REBO::SplineTdz_2(double x, double y, double z){

	int ix,iy,iz;
	double xx,xx2,xx3,yy,yy2,yy3,zz,zz2,zz3;
	double hx0, hx1,hy0, hy1,hz0, hz1;
	double Ret,temp0,temp1;
		
	ix = (int)(x);
	iy = (int)(y);
	iz = (int)(z);
	Ret = 0.0;
	
	if (iz >= SP_TMAXZ) {

	}else{
		zz = z - (double)(iz);
		zz2 = zz * zz;
		zz3 = zz2 * zz;
		hz0 = - 6.0 * zz + 6.0 * zz2;
		hz1 = 6.0 * zz - 6.0 * zz2;
		

		if (ix >= SP_TMAXX) {
	
			if (iy >= SP_TMAXY) {
	
				Ret = g_spT[SP_TMAXX][SP_TMAXY][iz]*hz0;
				Ret = Ret + g_spT[SP_TMAXX][SP_TMAXY][iz+1]*hz1;
			}else{

				yy = y - (double)(iy);
				yy2 = yy * yy;
				yy3 = yy2 * yy;
				hy0 = 1.0 - 3.0 * yy2 + 2.0 * yy3;
				hy1 = 3.0 * yy2 - 2.0 * yy3;
				
				temp0 = g_spT[SP_TMAXX][iy][iz]*hy0;
				temp0 = temp0 + g_spT[SP_TMAXX][iy+1][iz]*hy1;
				temp0 = temp0 * hz0;
					
				temp1 = g_spT[SP_TMAXX][iy][iz+1]*hy0;
				temp1 = temp1 + g_spT[SP_TMAXX][iy+1][iz+1]*hy1;
				temp1 = temp1 * hz1;
					
				Ret = Ret + temp0 + temp1;
			}
		}else{
			xx = x - (double)(ix);
			xx2 = xx * xx;
			xx3 = xx2 * xx;
			hx0 = 1.0 - 3.0 * xx2 + 2.0 * xx3;	//h(x=0);
			hx1 = 3.0 * xx2 - 2.0 * xx3;			//h(x=1);
			
			if (iy >= SP_TMAXY) {
	
				temp0 = g_spT[ix][SP_TMAXY][iz]*hx0;
				temp0 = temp0 + g_spT[ix+1][SP_TMAXY][iz]*hx1;
				temp0 = temp0 * hz0;
					
				temp1 = g_spT[ix][SP_TMAXY][iz+1]*hx0;
				temp1 = temp1 + g_spT[ix+1][SP_TMAXY][iz+1]*hx1;
				temp1 = temp1 * hz1;
					
				Ret = Ret + temp0 + temp1;
				
			}else{
				
				yy = y - (double)(iy);
				yy2 = yy * yy;
				yy3 = yy2 * yy;
				hy0 = 1.0 - 3.0 * yy2 + 2.0 * yy3;
				hy1 = 3.0 * yy2 - 2.0 * yy3;
				
				
				temp0 = g_spT[ix][iy][iz]*hx0;
				temp0 = temp0 + g_spT[ix+1][iy][iz]*hx1;
				temp0 = temp0 * hy0;
				
				temp1 = g_spT[ix][iy+1][iz]*hx0;
				temp1 = temp1 + g_spT[ix+1][iy+1][iz]*hx1;
				temp1 = temp1 * hy1;
					
				Ret = Ret + (temp0 + temp1) * hz0;
				
				temp0 = g_spT[ix][iy][iz+1]*hx0;
				temp0 = temp0 + g_spT[ix+1][iy][iz+1]*hx1;
				temp0 = temp0 * hy0;
					
				temp1 = g_spT[ix][iy+1][iz+1]*hx0;
				temp1 = temp1 + g_spT[ix+1][iy+1][iz+1]*hx1;
				temp1 = temp1 * hy1;
			
				Ret = Ret + (temp0 + temp1) * hz1;
				
			}
		}
	}
	
	return Ret;
}

inline void REBO::CalcQ(double Mt_i, double *Q_i, double *Qd_i){
		
	double theta;
	const double coffmax = 3.7;
	const double coffmin = 3.2;
	
	if (Mt_i < coffmin) {
		*Q_i = 1.0;
		*Qd_i = 0.0;
	}else if (Mt_i < coffmax) {
		theta = PI * 2.0 * (Mt_i - coffmin);
		*Q_i = (1.0 + cos(theta)) * 0.5;
		*Qd_i = - PI * sin(theta);
	}else{
		*Q_i = 0.0;
		*Qd_i = 0.0;
	}
}

inline void REBO::CalcCos3_4(int i, int j, vec3d e_ij, double idr_ij, double* cosI, vec3d* cosI_dIK, vec3d* cosI_dIJ){
		
	// argument ---------------------;
		//integer,intent(in):: i,j		// particle index;
		//double,intent(out):: cosI(BOND_CNT);
		//double,intent(out):: cosI_dIK(0:2,BOND_CNT),cosI_dIJ(0:2,BOND_CNT);
		
		
	//local variable;
	int k,iprt;
	vec3d e_ik;
	double idr_ik;
		
	//start routine -------------------------
	for( iprt = 0; iprt < m_rp[i].NbCnt; iprt++){
			
		k = m_rp[i].NbPrt[iprt];
	//exclude j particle from sumation;
		if (k == j) continue;
		
		idr_ik = m_rp[i].idr[iprt];
		e_ik = m_rp[i].e[iprt];
		
		cosI[iprt] = e_ij * e_ik;

//			cosI_dIK[iprt].x = (e_ij.x - cosI(iprt) * e_ik.x) * idr_ik;
//			cosI_dIK[iprt].y = (e_ij.y - cosI(iprt) * e_ik.y) * idr_ik;
//			cosI_dIK[iprt].z = (e_ij.z - cosI(iprt) * e_ik.z) * idr_ik;
		cosI_dIK[iprt] = (e_ij - e_ik * cosI[iprt]) * idr_ik;
		
//			cosI_dIJ[iprt].x = (e_ik.x - cosI(iprt) * e_ij.x) * idr_ij;
//			cosI_dIJ[iprt].y = (e_ik.y - cosI(iprt) * e_ij.y) * idr_ij;
//			cosI_dIJ[iprt].z = (e_ik.z - cosI(iprt) * e_ij.z) * idr_ij;
		cosI_dIJ[iprt] = (e_ik - e_ij * cosI[iprt]) * idr_ij;
		
	}
		
}
inline void REBO::CalcFor3Body(int i, int j, double* cosI, double* Gsum, double* Gary, double* Gdifary, double* Gsum2){
	
	//local variable;
		int k,iprt;
		double cos_ijk, Mt_i, Q_i, Qd_i;
		
		double Gc, GMc,Gcdif, GMcdif, eL;
		
		double fco;
		
	//parameter -----------------;
		const double cos109 = -0.333411977216052;			//dcos(6.082rad);
		const double cos120 = -0.5;
		double expLambda[4] = {1.0, 1.0, 1.0, 54.5981500331442};
		
	//start routine -------------------------;
	
		
	//bond angle term;
		
		Mt_i = m_rp[i].Nc + m_rp[i].Nh;
		CalcQ(Mt_i,&Q_i,&Qd_i);
		
		
		*Gsum = 0.0;
		*Gsum2 = 0.0;
		
		for(iprt = 0; iprt < m_rp[i].NbCnt; iprt++){
			
			k = m_rp[i].NbPrt[iprt];
		//exclude j particle from sumation;
			if (k == j) continue;
			
			fco = m_rp[i].fc[iprt];
			
			cos_ijk = cosI[iprt];
			
			eL = expLambda[m_knd[i] + m_knd[j] + m_knd[k]];
			
			if (m_knd[i] == KIND_C) {
				if (cos_ijk > cos109) {
					if (Q_i == 1.0) {
						subG(cos_ijk,2,&Gary[iprt]);
						subGdif(cos_ijk,2,&Gdifary[iprt]);

					}else if(Q_i == 0.0){
						subG(cos_ijk,1,&Gary[iprt]);
						subGdif(cos_ijk,1,&Gdifary[iprt]);

					}else{
						subG(cos_ijk,1,&Gc);
						subGdif(cos_ijk,1,&Gcdif);
						subG(cos_ijk,2,&GMc);
						subGdif(cos_ijk,2,&GMcdif);
						
						Gary[iprt] = (1.0-Q_i)*Gc + Q_i*GMc;
						Gdifary[iprt] = (1.0-Q_i)*Gcdif + Q_i*GMcdif;
						*Gsum2 += (-Gc + GMc) * eL * fco;		////g_fc(g_NbLIdx(iprt,i));

					}
					
				}else{
					subG(cos_ijk,1,&Gary[iprt]);
					subGdif(cos_ijk,1,&Gdifary[iprt]);

				}
			}else{	//hydrogen;
				Gary[iprt] = CalcG(cos_ijk);
				Gdifary[iprt] = CalcGdif(cos_ijk);
			}
			
			Gary[iprt] *= eL;
			Gdifary[iprt] *= eL;
			
			*Gsum += Gary[iprt] * fco;		////g_fc(g_NbLIdx(iprt,i));
		
		}
		
		*Gsum2 *= Qd_i;
}


inline void REBO::Force3Body_2(int i, int j, double V_A, double Bij, vec3d* cosI_dIK, vec3d* cosI_dIJ, double* Gary, double* Gdifary, double Gsum2, int iprt){
		
	double VB3,coef3,coef32;
	int k,prt3;
	vec3d e_ik;

	
	VB3 = - 0.25 * V_A * (Bij*Bij*Bij);
		
	// for ij,ik;
	for( prt3 = 0; prt3 < m_rp[i].NbCnt; prt3++){
		k = m_rp[i].NbPrt[prt3];
			
		//exclude j particle from sumation;
		if (k == j) continue;

		coef3 = VB3 * Gdifary[prt3] * m_rp[i].fc[prt3];		////g_fc(LIdx);
			
		m_rp[i].fij[prt3] += cosI_dIK[prt3] * coef3;

		// ij------	;
		
		m_rp[i].fij[iprt] += cosI_dIJ[prt3] * coef3;

	}
	
	for( prt3 = 0; prt3 < m_rp[i].NbCnt; prt3++){
			k = m_rp[i].NbPrt[prt3];

			e_ik = m_rp[i].e[prt3];
					
			if (k == j) {
				if (Gsum2 == 0.0) continue;
				coef32 = VB3 * Gsum2 * m_rp[i].dfc[prt3];
			}else{
				coef32 = VB3 * (Gsum2 + Gary[prt3]) * m_rp[i].dfc[prt3];
			}
			
			m_rp[i].fij[prt3] += e_ik * coef32;
	}
	
}


inline void REBO::ForceSplineP(int i, int j, double V_A, double Bij, double Nh_i, double Nc_i){
	

		
	double VB3,coef;
	int k,prt3;
	vec3d e_ik;
	double spPdx, spPdy;
	//double SplinePdx_2,SplinePdy_2;
	
	VB3 = - 0.25 * V_A * (Bij*Bij*Bij);
		
	if (m_knd[i] == KIND_C) {
		spPdx = SplinePdx_2(Nh_i, Nc_i,m_knd[j]);
		spPdy = SplinePdy_2(Nh_i, Nc_i,m_knd[j]);
		if ((spPdx==0.0) && (spPdy==0.0)) return;
		
		for( prt3 = 0; prt3 < m_rp[i].NbCnt; prt3++){
			k = m_rp[i].NbPrt[prt3];
			
			if (k == j) continue;

			e_ik = m_rp[i].e[prt3];
					
			coef = VB3 * m_rp[i].dfc[prt3];
			if (m_knd[k] == KIND_H) {
				coef=coef*spPdx;
			}else if (m_knd[k] == KIND_C) {
				coef=coef*spPdy;
			}else{
				continue;
			}
			
			m_rp[i].fij[prt3] += e_ik * coef;
		}
	}
}

//======================================================================;
//;
//		function CalcFor4Body;
//;
//			This routine calculate coefficient Bij that depend on;
//			local coordination and bond angle.;
//;
//;
//									made by Atsushi I.	2004/12/28;
//======================================================================;

inline void REBO::CalcFor4Body(int i, int j, double* cosI, double* cosJ, double* cosDH2sum, double* cosDH){
		
	//local variable;
	int k,iprt,l,jprt;
	double cos_kijl;
	vec3d e_ik;
	vec3d e_jl;
	double res;
	
	//start routine -------------------------;
	*cosDH2sum = 0.0;
		
	//bond angle term;
	
	for( iprt = 0; iprt < m_rp[i].NbCnt; iprt++){
		k = m_rp[i].NbPrt[iprt];
		//exclude j particle from sumation;
		if (k == j) continue;
		
		e_ik = m_rp[i].e[iprt];
					
		for( jprt = 0; jprt < m_rp[j].NbCnt; jprt++){
				
			l = m_rp[j].NbPrt[jprt];
			//exclude i particle from sumation;
			if (l == i) continue;
				
			e_jl = m_rp[j].e[jprt];

			res = e_ik * e_jl;
			cos_kijl = -res - cosI[iprt]*cosJ[jprt];
			cosDH[iprt* BOND_CNT + jprt] = cos_kijl;
				
			res = 1.00 - cos_kijl*cos_kijl;
			*cosDH2sum += res * m_rp[i].fc[iprt] * m_rp[j].fc[jprt];
		}
	}
	
}


inline void REBO::Force4Body_2(int i,int j, double V_A, double T_ij, double* cosI,vec3d* cosI_dIK,vec3d* cosI_dIJ,double* cosJ,vec3d* cosJ_dJL,vec3d* cosJ_dJI,double* cosDH, int ijprt){

		
	//local variable;
	int k,iprt,l,jprt;
	vec3d e_ik;
	double idr_ik;
	vec3d e_jl;
	double idr_jl;
	double cos_ik_jl;
	double coef4;
	vec3d Pxyz;

	double fco_ik, fco_jl, dfco_ik, dfco_jl;
	
	if (T_ij != 0.0) {
		for( iprt = 0; iprt < m_rp[i].NbCnt; iprt++){
			k = m_rp[i].NbPrt[iprt];

			//exclude j particle from sumation;
			if (k == j) continue;

			fco_ik = m_rp[i].fc[iprt];
			dfco_ik = m_rp[i].dfc[iprt];
	
			idr_ik = m_rp[i].idr[iprt];

			e_ik = m_rp[i].e[iprt];
				

			for( jprt = 0; jprt < m_rp[j].NbCnt; jprt++){
			
				l = m_rp[j].NbPrt[jprt];
				if (l == i) continue;
			
				fco_jl = m_rp[j].fc[jprt];
				dfco_jl = m_rp[j].dfc[jprt];
			
				idr_jl = m_rp[j].idr[jprt];

				e_jl = m_rp[j].e[jprt];

				coef4 = 2.0 * V_A * T_ij * cosDH[iprt*BOND_CNT + jprt] * fco_ik * fco_jl;
			
				cos_ik_jl = e_jl * e_ik;
				Pxyz = (e_jl - e_ik*cos_ik_jl) * idr_ik;
				
				
				m_rp[i].fij[iprt] += (Pxyz + cosI_dIK[iprt] * cosJ[jprt]) * coef4;

				
				Pxyz = (e_ik - e_jl * cos_ik_jl ) * idr_jl;
			
				m_rp[j].fij[jprt] +=  (Pxyz + cosJ_dJL[jprt] * cosI[iprt]) * coef4;

				
				m_rp[i].fij[ijprt] += (cosI_dIJ[iprt]*cosJ[jprt] - cosJ_dJI[jprt]*cosI[iprt]) * coef4;

				//by differencial cutoff function;
			// ik-----;
				coef4 = V_A * T_ij * (1.0 - (cosDH[iprt*BOND_CNT + jprt]*cosDH[iprt*BOND_CNT + jprt]));
				//coef4 *= dfco_ik * fco_jl;
			
				m_rp[i].fij[iprt] += e_ik * (coef4*dfco_ik * fco_jl);

			
				//coef4 = V_A * T_ij * (1.0 - (cosDH[iprt*BOND_CNT + jprt]*cosDH[iprt*BOND_CNT + jprt]));
				//coef4 *= fco_ik * dfco_jl;

				m_rp[j].fij[jprt] += e_jl * (coef4*fco_ik * dfco_jl);
			}
		}
	}
	
}

	
inline void REBO::CalcForConjugation(int i, int j, double* retSsum, double* S_ki, double* dS_ki){
	
	//local variable -----------------------------;
		int iprt,k;
		double Nt_ki,agl;
		double Ssum,ski,dski;
		double fco;
		
	//parameter ----------------------------;
		//for conjugate;
		const double Xmax = 3.0;
		const double Xmin = 2.0;
		
	//start routine ------------------------;
	
		//calculate conjugation;
		Ssum = 0.0;
		for( iprt = 0; iprt < m_rp[i].NbCnt; iprt++){
		
			k = m_rp[i].NbPrt[iprt];
		//exclude j particle from sumation;
			if (k == j) continue;
		//sum of only carbon;
			if (m_knd[k] != KIND_C) continue;
			
////			kl = g_NbLIdx(iprt,i);
			fco = m_rp[i].fc[iprt];
			Nt_ki = m_rp[k].Nc + m_rp[k].Nh - fco;		////g_fc(kl);
			

			if (Nt_ki <= Xmin) {
				ski = 1.0;
				dski = 0.0;
			}else if (Nt_ki >= Xmax) {
				ski = 0.0;
				dski = 0.0;
			}else{
				agl = PI * ( Nt_ki - Xmin); // / (Xmax - Xmin) ;
				ski = 0.5 * (1.0 + cos(agl));
				dski = -0.5 * PI * sin(agl);
			}

			Ssum = Ssum + fco * ski;
			S_ki[iprt] = ski;
			dS_ki[iprt] = dski;
		}
		
		*retSsum = Ssum;
}


inline void REBO::ForcePiBond(int i, int j,double V_A,double Nt_ij,double Nt_ji,double Nconj,double Ssum_i,double *S_ki,double *dS_ki,double Ssum_j,double *S_lj,double *dS_lj){
		

	double coef;
	int k,prt3,s,prt4;
	vec3d exyz;
	double spFdx, spFdy, spFdz;
	double VF;
	double fco3, dfco3, dfco4;
	
	spFdx = SplineFdx_2(Nt_ij,Nt_ji,Nconj,m_knd[i] + m_knd[j]);
	spFdy = SplineFdx_2(Nt_ji,Nt_ij,Nconj,m_knd[i] + m_knd[j]);
	spFdz = SplineFdz_2(Nt_ij,Nt_ji,Nconj,m_knd[i] + m_knd[j]);
	
	if (spFdx != 0.0) {
		for( prt3 = 0; prt3 < m_rp[i].NbCnt; prt3++){
			k = m_rp[i].NbPrt[prt3];

			if (k == j) continue;
			
			dfco3 = m_rp[i].dfc[prt3];
			
			exyz = m_rp[i].e[prt3];
			
			coef = V_A * spFdx * dfco3;
				
			m_rp[i].fij[prt3] += exyz * coef;
		}
	}
	
	if (spFdy != 0.0) {
		for( prt3 = 0; prt3 < m_rp[j].NbCnt; prt3++){
			k = m_rp[j].NbPrt[prt3];

			if (k == i) continue;
			
			dfco3 = m_rp[j].dfc[prt3];
			
			
			exyz = m_rp[j].e[prt3];

			coef = V_A * spFdy * dfco3;
		
			m_rp[j].fij[prt3] += exyz * coef;
		}
	}
	
	if (Ssum_i != 0.0) {
		for( prt3 = 0; prt3 < m_rp[i].NbCnt; prt3++){
			k = m_rp[i].NbPrt[prt3];

			if (k == j) continue;
			if (m_knd[k] != KIND_C) continue;
			
			fco3 = m_rp[i].fc[prt3];
			dfco3 = m_rp[i].dfc[prt3];
			
			exyz = m_rp[i].e[prt3];
					
			coef = V_A * spFdz * S_ki[prt3] * dfco3;
				
			m_rp[i].fij[prt3] += exyz * coef;


			if (dS_ki[prt3] == 0.0) continue;

			VF = V_A * spFdz * dS_ki[prt3] * fco3; // * 2.0 * Ssum_i;
			for( prt4 = 0;prt4 < m_rp[k].NbCnt; prt4++){
				s = m_rp[k].NbPrt[prt4];

				if (s == i) continue;

				dfco4 = m_rp[k].dfc[prt4];
				
				exyz = m_rp[k].e[prt4];
						
				coef = VF * dfco4;

				
				m_rp[k].fij[prt4] += exyz * coef;
			}
		}
	}
	
	if(Ssum_j != 0.0) {
		for( prt3 = 0; prt3 < m_rp[j].NbCnt; prt3 ++){
			k = m_rp[j].NbPrt[prt3];

			if (k == i) continue;
			if (m_knd[k]!=KIND_C) continue;
			
			fco3 = m_rp[j].fc[prt3];
			dfco3 = m_rp[j].dfc[prt3];

			exyz = m_rp[j].e[prt3];
			coef = V_A * spFdz *  S_lj[prt3] * dfco3;
				
			m_rp[j].fij[prt3] += exyz * coef;

			if (dS_lj[prt3] == 0.0) continue;

			VF = V_A * spFdz * dS_lj[prt3] * fco3; // * 2.0 * Ssum_j;
			for( prt4 = 0; prt4 < m_rp[k].NbCnt; prt4++){
				s = m_rp[k].NbPrt[prt4];

				if (s == j) continue;

				dfco4 = m_rp[k].dfc[prt4];
				
				exyz = m_rp[k].e[prt4];

				coef = VF * dfco4;

				m_rp[k].fij[prt4] += exyz * coef;
				
			}
		}
	}
	
}



inline void REBO::ForceSplineT(int i, int j, double V_A, double Nt_ij, double Nt_ji, double Nconj, double cosDH2sum, double Ssum_i, double *S_ki, double *dS_ki, double Ssum_j, double *S_lj, double *dS_lj){
	

	double coef;
	int k,prt3,s,prt4;
	vec3d exyz;
	double spTdx, spTdy, spTdz;
	double VF;
	double fco3, dfco3, dfco4;
	
	
	if (cosDH2sum==0.0) return;
	if (m_knd[i]+m_knd[j] != KIND_CC) return;
	
	spTdx = SplineTdx_2(Nt_ij,Nt_ji,Nconj);//,m_knd(i) + m_knd(j));
	spTdy = SplineTdx_2(Nt_ji,Nt_ij,Nconj);//,m_knd(i) + m_knd(j));
	spTdz = SplineTdz_2(Nt_ij,Nt_ji,Nconj);//,m_knd(i) + m_knd(j));
	
	if (spTdx != 0.0) {
		for( prt3 = 0; prt3 < m_rp[i].NbCnt; prt3++){
			k = m_rp[i].NbPrt[prt3];

			if (k == j) continue;
		
			dfco3 = m_rp[i].dfc[prt3];
		
			exyz = m_rp[i].e[prt3];
			
			coef = V_A * spTdx * cosDH2sum * dfco3;
			
			m_rp[i].fij[prt3] += exyz * coef;

		}
	}

	if (spTdy != 0.0) {
		for( prt3 = 0; prt3 < m_rp[j].NbCnt; prt3++){
			k = m_rp[j].NbPrt[prt3];

			if (k == i) continue;
			
			dfco3 = m_rp[j].dfc[prt3];
			
			exyz = m_rp[j].e[prt3];
					
			coef = V_A * spTdy * cosDH2sum * dfco3;
			
			m_rp[j].fij[prt3] += exyz * coef;
		}
	}
	
	if (spTdz != 0.0) {
		if (Ssum_i != 0.0) {
			for( prt3 = 0; prt3 < m_rp[i].NbCnt; prt3++){
				k = m_rp[i].NbPrt[prt3];

				if (k == j) continue;
			
				if (m_knd[k] !=KIND_C) continue;
			
				fco3 = m_rp[i].fc[prt3];
				dfco3 = m_rp[i].dfc[prt3];
				
				exyz = m_rp[i].e[prt3];
					
				coef = V_A * cosDH2sum * spTdz * S_ki[prt3] * dfco3;
				
				m_rp[i].fij[prt3] += exyz * coef;


				if (dS_ki[prt3] == 0.0) continue;

				VF = V_A * cosDH2sum * spTdz * fco3 * dS_ki[prt3]; // * 2.0 * Ssum_i;
				for( prt4 = 0; prt4 < m_rp[k].NbCnt; prt4++){
					s = m_rp[k].NbPrt[prt4];

					if (s == i) continue;

					dfco4 = m_rp[k].dfc[prt4];
				
					exyz = m_rp[k].e[prt4];
					
					coef = VF * dfco4;
					
					m_rp[k].fij[prt4] += exyz * coef;
				}
			}
		}

		if(Ssum_j != 0.0) {
			for( prt3 = 0; prt3 < m_rp[j].NbCnt; prt3 ++){
				k = m_rp[j].NbPrt[prt3];

				if (k == i) continue;
			
				if (m_knd[k] != KIND_C) continue;
			
				fco3 = m_rp[j].fc[prt3];
				dfco3 = m_rp[j].dfc[prt3];
					
				exyz = m_rp[j].e[prt3];
				
				coef = V_A * cosDH2sum * spTdz * S_lj[prt3] * dfco3;
			
				m_rp[j].fij[prt3] += exyz * coef;
				
				if (dS_lj[prt3] == 0.0) continue;

				VF = V_A * cosDH2sum * spTdz * fco3 * dS_lj[prt3]; // * 2.0 * Ssum_j;
	
				for( prt4 = 0; prt4 < m_rp[k].NbCnt; prt4++){
					s = m_rp[k].NbPrt[prt4];

					if (s == j) continue;

					dfco4 = m_rp[k].dfc[prt4];
					
					exyz = m_rp[k].e[prt4];
					
					coef = VF * dfco4;
				
					m_rp[k].fij[prt4] += exyz * coef;

				}
			}
		}
	}
}

inline void REBO::CalcF(){		

	int i, iprt, j, Lidx,jprt;
	vec3d frc;
		
	for( Lidx = 0; Lidx < m_bondCnt; Lidx++){
			
			i = m_bonding[Lidx].i;
			j = m_bonding[Lidx].j;
		
			iprt = m_bonding[Lidx].bondnum_i;
			jprt = m_bonding[Lidx].bondnum_j;
			
			frc = m_rp[i].fij[iprt] - m_rp[j].fij[jprt];
				
			m_f[i] +=  frc;
			m_f[j] -=  frc;
				
	}
}


inline void REBO::CalcFJ(){
		

	int i, iprt, j, Lidx,jprt;
	double J_ij;
	vec3d frc;
		
	for( Lidx = 0; Lidx < m_bondCnt; Lidx++){
			
			i = m_bonding[Lidx].i;
			j = m_bonding[Lidx].j;
		
			iprt = m_bonding[Lidx].bondnum_i;
			jprt = m_bonding[Lidx].bondnum_j;
			
			frc = m_rp[i].fij[iprt] - m_rp[j].fij[jprt];
				
			m_f[i] +=  frc;
			m_f[j] -=  frc;
				
			//energy current;
			J_ij = - (m_p[i]/m_mass[i] + m_p[j]/m_mass[j]) * frc;
				//J_ij = - (m_p[i] + m_p[j]) * frc;
				//J_ij /= (m_mass[i] + m_mass[j]);

				m_Jij[Lidx] +=  J_ij;
	}
}

inline void REBO::CalcFJFrv(){
		

	int i, iprt, j, Lidx,jprt;
	double J_ij;
	vec3d frc;
	vec3d Fr_v;
	m_Fr_v.Set(0.0, 0.0, 0.0);
		
	for( Lidx = 0; Lidx < m_bondCnt; Lidx++){
			
			i = m_bonding[Lidx].i;
			j = m_bonding[Lidx].j;
		
			iprt = m_bonding[Lidx].bondnum_i;
			jprt = m_bonding[Lidx].bondnum_j;
			
			frc = m_rp[i].fij[iprt] - m_rp[j].fij[jprt];
				
			m_f[i] +=  frc;
			m_f[j] -=  frc;

			// \sum (r * F)//////////////////////
			Fr_v.x = (m_rp[i].e[iprt].x * frc.x) * m_rp[i].dr[iprt];
			Fr_v.y = (m_rp[i].e[iprt].y * frc.y) * m_rp[i].dr[iprt];
			Fr_v.z = (m_rp[i].e[iprt].z * frc.z) * m_rp[i].dr[iprt];
			m_Fr_v += Fr_v;

				
			//energy current;
			J_ij = - (m_p[i]/m_mass[i] + m_p[j]/m_mass[j]) * frc;
				//J_ij = - (m_p[i] + m_p[j]) * frc;
				//J_ij /= (m_mass[i] + m_mass[j]);

			m_Jij[Lidx] +=  J_ij;
	}
}
	
inline void REBO::CalcFJFrm(){
		

	int i, iprt, j, Lidx,jprt;
	double J_ij;
	vec3d frc;
	mat33d Fr_m;
	m_Fr_m.Set(0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0);
		
	for( Lidx = 0; Lidx < m_bondCnt; Lidx++){
			
			i = m_bonding[Lidx].i;
			j = m_bonding[Lidx].j;
		
			iprt = m_bonding[Lidx].bondnum_i;
			jprt = m_bonding[Lidx].bondnum_j;
			
			frc = m_rp[i].fij[iprt] - m_rp[j].fij[jprt];
				
			m_f[i] +=  frc;
			m_f[j] -=  frc;

			// tensor (r x F)//////////////////////
			Fr_m.a = frc * m_rp[i].e[iprt].x * m_rp[i].dr[iprt];
			Fr_m.b = frc * m_rp[i].e[iprt].y * m_rp[i].dr[iprt];
			Fr_m.c = frc * m_rp[i].e[iprt].z * m_rp[i].dr[iprt];
			m_Fr_m += Fr_m;

				
			//energy current;
			J_ij = - (m_p[i]/m_mass[i] + m_p[j]/m_mass[j]) * frc;
				//J_ij = - (m_p[i] + m_p[j]) * frc;
				//J_ij /= (m_mass[i] + m_mass[j]);

				m_Jij[Lidx] +=  J_ij;
	}
}

double REBO::GetPotentialEnergy(int count){
		
		int i;
		double sum;
		
		sum = 0.0;
		for( i = 0; i < count; i++){
			sum += m_pot[i];
		}
		return sum * 0.5;

}

void REBO::GetFr(double* a_Fr){
		
		*a_Fr = m_Fr_v.x + m_Fr_v.y + m_Fr_v.z;

}

void REBO::GetFrv3d(vec3d* a_Fr_v){
		*a_Fr_v = m_Fr_v;

}

void REBO::GetFrm33d(mat33d* a_Fr_m){
		*a_Fr_m = m_Fr_m;

}

void REBO::GetBondingPointers(int* bcnt, REBO_BONDING** bonding){
	*bcnt = m_bondCnt;
	//*pli = g_li;
	//*plj = g_lj;
	*bonding = m_bonding;
}

void REBO::GetREBOPointers(REBO_PARTICLE** prp){
	*prp = m_rp;
}